"""NAS 共享存储配置 - 解决集群环境文件一致性问题"""
import os
import json
import shutil
from pathlib import Path
from datetime import datetime
from typing import Dict, Optional, Any
import fcntl
import hashlib
import time


class NASStorage:
    """NAS 存储管理器 - 确保所有节点访问同一份数据"""
    
    def __init__(self, nas_base_path: str = "/app/mcp-servers/mcp-servers/html_agent"):
        """
        初始化 NAS 存储
        
        Args:
            nas_base_path: NAS 挂载路径
        """
        self.nas_base = Path(nas_base_path)
        self.projects_dir = self.nas_base / "projects"
        self.cache_dir = self.nas_base / "cache"
        self.locks_dir = self.nas_base / "locks"
        self.metadata_dir = self.nas_base / "metadata"
        
        # 创建必要的目录结构
        self._init_directories()
        
    def _init_directories(self):
        """初始化目录结构"""
        for dir_path in [
            self.projects_dir,
            self.cache_dir,
            self.locks_dir,
            self.metadata_dir,
            self.cache_dir / "plans",
            self.cache_dir / "contexts",
            self.cache_dir / "jobs"
        ]:
            dir_path.mkdir(parents=True, exist_ok=True)
    
    def get_project_path(self, project_name: str, create: bool = True) -> Path:
        """
        获取项目路径
        
        Args:
            project_name: 项目名称
            create: 是否创建目录
            
        Returns:
            项目完整路径
        """
        # 生成唯一的项目目录名
        timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
        safe_name = "".join(c for c in project_name if c.isalnum() or c in ('-', '_'))
        project_dir = self.projects_dir / f"{safe_name}_{timestamp}"
        
        if create:
            project_dir.mkdir(parents=True, exist_ok=True)
            # 创建项目元数据
            self._save_project_metadata(project_dir, project_name)
            
        return project_dir
    
    def _save_project_metadata(self, project_dir: Path, project_name: str):
        """保存项目元数据"""
        metadata = {
            "project_name": project_name,
            "project_path": str(project_dir),
            "created_at": datetime.now().isoformat(),
            "node_id": os.environ.get("NODE_ID", "unknown"),
            "server_instance": os.environ.get("HOSTNAME", "unknown")
        }
        
        metadata_file = self.metadata_dir / f"{project_dir.name}.json"
        with open(metadata_file, 'w', encoding='utf-8') as f:
            json.dump(metadata, f, ensure_ascii=False, indent=2)
    
    def save_plan(self, plan_id: str, plan_data: Dict[str, Any]) -> Path:
        """
        保存计划到 NAS
        
        Args:
            plan_id: 计划 ID
            plan_data: 计划数据
            
        Returns:
            保存的文件路径
        """
        plan_path = self.cache_dir / "plans" / f"{plan_id}.json"
        
        # 添加时间戳和节点信息
        plan_data["saved_at"] = datetime.now().isoformat()
        plan_data["node_id"] = os.environ.get("NODE_ID", "unknown")
        
        # 使用文件锁确保原子写入
        with self._file_lock(plan_path):
            with open(plan_path, 'w', encoding='utf-8') as f:
                json.dump(plan_data, f, ensure_ascii=False, indent=2)
        
        return plan_path
    
    def load_plan(self, plan_id: str) -> Optional[Dict[str, Any]]:
        """
        从 NAS 加载计划
        
        Args:
            plan_id: 计划 ID
            
        Returns:
            计划数据，如果不存在则返回 None
        """
        plan_path = self.cache_dir / "plans" / f"{plan_id}.json"
        
        if not plan_path.exists():
            return None
            
        with self._file_lock(plan_path, exclusive=False):
            with open(plan_path, 'r', encoding='utf-8') as f:
                return json.load(f)
    
    def save_job_state(self, job_id: str, job_data: Dict[str, Any]) -> Path:
        """
        保存任务状态到 NAS
        
        Args:
            job_id: 任务 ID
            job_data: 任务数据
            
        Returns:
            保存的文件路径
        """
        job_path = self.cache_dir / "jobs" / f"{job_id}.json"
        
        # 添加更新时间
        job_data["updated_at"] = datetime.now().isoformat()
        job_data["node_id"] = os.environ.get("NODE_ID", "unknown")
        
        with self._file_lock(job_path):
            with open(job_path, 'w', encoding='utf-8') as f:
                json.dump(job_data, f, ensure_ascii=False, indent=2)
        
        return job_path
    
    def load_job_state(self, job_id: str) -> Optional[Dict[str, Any]]:
        """
        从 NAS 加载任务状态
        
        Args:
            job_id: 任务 ID
            
        Returns:
            任务数据，如果不存在则返回 None
        """
        job_path = self.cache_dir / "jobs" / f"{job_id}.json"
        
        if not job_path.exists():
            return None
            
        with self._file_lock(job_path, exclusive=False):
            with open(job_path, 'r', encoding='utf-8') as f:
                return json.load(f)
    
    def list_projects(self, limit: int = 20) -> list:
        """
        列出最近的项目
        
        Args:
            limit: 返回的项目数量
            
        Returns:
            项目列表
        """
        projects = []
        
        # 读取所有元数据文件
        for metadata_file in self.metadata_dir.glob("*.json"):
            try:
                with open(metadata_file, 'r', encoding='utf-8') as f:
                    project_info = json.load(f)
                    projects.append(project_info)
            except Exception:
                continue
        
        # 按创建时间排序
        projects.sort(key=lambda x: x.get("created_at", ""), reverse=True)
        
        return projects[:limit]
    
    def _file_lock(self, file_path: Path, exclusive: bool = True):
        """
        文件锁上下文管理器
        
        Args:
            file_path: 文件路径
            exclusive: 是否独占锁
            
        Returns:
            锁对象
        """
        # 使用单独的锁文件
        lock_file = self.locks_dir / f"{file_path.name}.lock"
        
        class FileLock:
            def __init__(self, lock_path: Path, exclusive: bool):
                self.lock_path = lock_path
                self.exclusive = exclusive
                self.lock_fd = None
                
            def __enter__(self):
                self.lock_path.parent.mkdir(parents=True, exist_ok=True)
                self.lock_fd = open(self.lock_path, 'w')
                
                # 获取文件锁
                flag = fcntl.LOCK_EX if self.exclusive else fcntl.LOCK_SH
                
                # 重试机制
                max_retries = 10
                for i in range(max_retries):
                    try:
                        fcntl.flock(self.lock_fd, flag | fcntl.LOCK_NB)
                        break
                    except IOError:
                        if i == max_retries - 1:
                            raise
                        time.sleep(0.1 * (i + 1))  # 指数退避
                
                return self
                
            def __exit__(self, exc_type, exc_val, exc_tb):
                if self.lock_fd:
                    fcntl.flock(self.lock_fd, fcntl.LOCK_UN)
                    self.lock_fd.close()
                    
                # 清理过期的锁文件
                try:
                    if self.lock_path.exists():
                        # 如果锁文件超过1小时，删除它
                        if time.time() - self.lock_path.stat().st_mtime > 3600:
                            self.lock_path.unlink(missing_ok=True)
                except Exception:
                    pass
        
        return FileLock(lock_file, exclusive)
    
    def generate_unique_id(self, prefix: str = "") -> str:
        """
        生成唯一 ID（结合时间戳、节点 ID 和随机数）
        
        Args:
            prefix: ID 前缀
            
        Returns:
            唯一 ID
        """
        node_id = os.environ.get("NODE_ID", "default")
        timestamp = datetime.now().strftime("%Y%m%d%H%M%S%f")
        random_str = hashlib.md5(os.urandom(16)).hexdigest()[:8]
        
        if prefix:
            return f"{prefix}_{node_id}_{timestamp}_{random_str}"
        return f"{node_id}_{timestamp}_{random_str}"
    
    def cleanup_old_projects(self, days_to_keep: int = 7) -> int:
        """
        清理旧项目
        
        Args:
            days_to_keep: 保留天数
            
        Returns:
            删除的项目数量
        """
        deleted_count = 0
        cutoff_time = time.time() - (days_to_keep * 24 * 3600)
        
        for project_dir in self.projects_dir.iterdir():
            if project_dir.is_dir():
                try:
                    # 检查修改时间
                    if project_dir.stat().st_mtime < cutoff_time:
                        shutil.rmtree(project_dir)
                        
                        # 删除对应的元数据
                        metadata_file = self.metadata_dir / f"{project_dir.name}.json"
                        metadata_file.unlink(missing_ok=True)
                        
                        deleted_count += 1
                except Exception:
                    continue
        
        return deleted_count


# 全局 NAS 存储实例
_nas_storage: Optional[NASStorage] = None


def get_nas_storage() -> NASStorage:
    """获取 NAS 存储实例（单例）"""
    global _nas_storage
    if _nas_storage is None:
        nas_path = os.environ.get(
            "NAS_STORAGE_PATH", 
            "/app/mcp-servers/mcp-servers/html_agent"
        )
        _nas_storage = NASStorage(nas_path)
    return _nas_storage


# 导出的便捷函数
def save_project_file(project_name: str, relative_path: str, content: str) -> str:
    """
    保存项目文件到 NAS
    
    Args:
        project_name: 项目名称
        relative_path: 相对路径
        content: 文件内容
        
    Returns:
        保存的完整路径
    """
    storage = get_nas_storage()
    project_dir = storage.get_project_path(project_name)
    
    file_path = project_dir / relative_path
    file_path.parent.mkdir(parents=True, exist_ok=True)
    
    with open(file_path, 'w', encoding='utf-8') as f:
        f.write(content)
    
    return str(file_path)


def load_project_file(project_path: str, relative_path: str) -> Optional[str]:
    """
    从 NAS 加载项目文件
    
    Args:
        project_path: 项目路径
        relative_path: 相对路径
        
    Returns:
        文件内容，如果不存在则返回 None
    """
    file_path = Path(project_path) / relative_path
    
    if not file_path.exists():
        return None
    
    with open(file_path, 'r', encoding='utf-8') as f:
        return f.read()


"""进度跟踪模块 - 支持集群环境下的任务进度查询"""
import json
import time
import os
from pathlib import Path
from typing import Dict, List, Optional, Any
from datetime import datetime
import hashlib


class ProgressTracker:
    """任务进度跟踪器 - 支持分布式环境"""
    
    def __init__(self, nas_base_path: str = "/app/mcp-servers/mcp-servers/html_agent"):
        """
        初始化进度跟踪器
        
        Args:
            nas_base_path: NAS 基础路径
        """
        self.nas_base = Path(nas_base_path)
        self.progress_dir = self.nas_base / "mcp_data" / "make_web" / "progress"
        self.progress_dir.mkdir(parents=True, exist_ok=True)
        
    def create_task(self, task_type: str, description: str) -> str:
        """
        创建新任务并返回任务 ID
        
        Args:
            task_type: 任务类型 (plan_site, execute_plan, etc.)
            description: 任务描述
            
        Returns:
            任务 ID
        """
        # 生成唯一任务 ID
        timestamp = datetime.now().strftime("%Y%m%d_%H%M%S_%f")
        node_id = os.environ.get("NODE_ID", "node0")
        random_suffix = hashlib.md5(f"{timestamp}{description}".encode()).hexdigest()[:8]
        task_id = f"{task_type}_{node_id}_{timestamp}_{random_suffix}"
        
        # 创建任务记录
        task_info = {
            "task_id": task_id,
            "task_type": task_type,
            "description": description,
            "status": "pending",
            "progress": 0,
            "created_at": datetime.now().isoformat(),
            "node_id": node_id,
            "events": [],
            "current_step": None,
            "total_steps": None
        }
        
        self._save_task_info(task_id, task_info)
        return task_id
    
    def update_progress(
        self, 
        task_id: str, 
        progress: int, 
        status: str = "running",
        message: str = None,
        current_step: int = None,
        total_steps: int = None,
        data: Dict[str, Any] = None
    ) -> None:
        """
        更新任务进度
        
        Args:
            task_id: 任务 ID
            progress: 进度百分比 (0-100)
            status: 任务状态 (pending/running/completed/failed)
            message: 进度消息
            current_step: 当前步骤
            total_steps: 总步骤数
            data: 附加数据
        """
        task_info = self._load_task_info(task_id)
        if not task_info:
            task_info = {"task_id": task_id, "events": []}
        
        # 更新基本信息
        task_info["progress"] = progress
        task_info["status"] = status
        task_info["updated_at"] = datetime.now().isoformat()
        
        if current_step is not None:
            task_info["current_step"] = current_step
        if total_steps is not None:
            task_info["total_steps"] = total_steps
            
        # 添加事件记录
        event = {
            "timestamp": datetime.now().isoformat(),
            "progress": progress,
            "status": status,
            "message": message
        }
        if data:
            event["data"] = data
            
        task_info["events"].append(event)
        
        # 保存更新
        self._save_task_info(task_id, task_info)
        
        # 同时写入流式日志（用于实时查询）
        self._append_progress_log(task_id, event)
    
    def get_task_progress(self, task_id: str) -> Optional[Dict[str, Any]]:
        """
        获取任务进度
        
        Args:
            task_id: 任务 ID
            
        Returns:
            任务进度信息
        """
        return self._load_task_info(task_id)
    
    def get_task_events(
        self, 
        task_id: str, 
        since_timestamp: Optional[str] = None
    ) -> List[Dict[str, Any]]:
        """
        获取任务事件（支持增量查询）
        
        Args:
            task_id: 任务 ID
            since_timestamp: 从此时间戳之后的事件
            
        Returns:
            事件列表
        """
        task_info = self._load_task_info(task_id)
        if not task_info:
            return []
        
        events = task_info.get("events", [])
        
        if since_timestamp:
            # 过滤出指定时间后的事件
            events = [
                e for e in events 
                if e.get("timestamp", "") > since_timestamp
            ]
        
        return events
    
    def list_tasks(
        self, 
        task_type: Optional[str] = None,
        status: Optional[str] = None,
        limit: int = 20
    ) -> List[Dict[str, Any]]:
        """
        列出任务列表
        
        Args:
            task_type: 筛选任务类型
            status: 筛选任务状态
            limit: 返回数量限制
            
        Returns:
            任务列表
        """
        tasks = []
        
        # 扫描所有任务文件
        for task_file in self.progress_dir.glob("*.json"):
            if task_file.name.endswith("_log.json"):
                continue  # 跳过日志文件
                
            try:
                with open(task_file, 'r', encoding='utf-8') as f:
                    task_info = json.load(f)
                    
                # 应用过滤条件
                if task_type and task_info.get("task_type") != task_type:
                    continue
                if status and task_info.get("status") != status:
                    continue
                    
                # 精简信息，不包含完整事件列表
                summary = {
                    "task_id": task_info.get("task_id"),
                    "task_type": task_info.get("task_type"),
                    "description": task_info.get("description"),
                    "status": task_info.get("status"),
                    "progress": task_info.get("progress"),
                    "created_at": task_info.get("created_at"),
                    "updated_at": task_info.get("updated_at"),
                    "current_step": task_info.get("current_step"),
                    "total_steps": task_info.get("total_steps")
                }
                tasks.append(summary)
                
            except Exception:
                continue
        
        # 按创建时间倒序排序
        tasks.sort(key=lambda x: x.get("created_at", ""), reverse=True)
        
        return tasks[:limit]
    
    def get_realtime_progress(self, task_id: str) -> Dict[str, Any]:
        """
        获取实时进度（用于 SSE 流式传输）
        
        Args:
            task_id: 任务 ID
            
        Returns:
            实时进度信息
        """
        task_info = self._load_task_info(task_id)
        if not task_info:
            return {
                "task_id": task_id,
                "status": "not_found",
                "progress": 0
            }
        
        # 计算详细进度
        progress_detail = {
            "task_id": task_id,
            "status": task_info.get("status", "unknown"),
            "progress": task_info.get("progress", 0),
            "current_step": task_info.get("current_step"),
            "total_steps": task_info.get("total_steps"),
            "message": None
        }
        
        # 获取最新消息
        events = task_info.get("events", [])
        if events:
            latest_event = events[-1]
            progress_detail["message"] = latest_event.get("message")
            progress_detail["last_update"] = latest_event.get("timestamp")
        
        # 如果有步骤信息，计算步骤进度
        if progress_detail["current_step"] and progress_detail["total_steps"]:
            step_progress = (progress_detail["current_step"] / progress_detail["total_steps"]) * 100
            progress_detail["step_progress"] = round(step_progress, 1)
        
        return progress_detail
    
    def _save_task_info(self, task_id: str, task_info: Dict[str, Any]) -> None:
        """保存任务信息到 NAS"""
        task_file = self.progress_dir / f"{task_id}.json"
        
        # 原子性写入（先写临时文件，再重命名）
        temp_file = task_file.with_suffix('.tmp')
        with open(temp_file, 'w', encoding='utf-8') as f:
            json.dump(task_info, f, ensure_ascii=False, indent=2)
        
        # 重命名（原子操作）
        temp_file.replace(task_file)
    
    def _load_task_info(self, task_id: str) -> Optional[Dict[str, Any]]:
        """从 NAS 加载任务信息"""
        task_file = self.progress_dir / f"{task_id}.json"
        
        if not task_file.exists():
            return None
            
        try:
            with open(task_file, 'r', encoding='utf-8') as f:
                return json.load(f)
        except Exception:
            return None
    
    def _append_progress_log(self, task_id: str, event: Dict[str, Any]) -> None:
        """追加进度日志（用于流式查询）"""
        log_file = self.progress_dir / f"{task_id}_log.jsonl"
        
        try:
            with open(log_file, 'a', encoding='utf-8') as f:
                f.write(json.dumps(event, ensure_ascii=False))
                f.write('\n')
        except Exception:
            pass
    
    def cleanup_old_tasks(self, days_to_keep: int = 7) -> int:
        """
        清理旧任务
        
        Args:
            days_to_keep: 保留天数
            
        Returns:
            清理的任务数量
        """
        cleaned = 0
        cutoff_time = time.time() - (days_to_keep * 24 * 3600)
        
        for task_file in self.progress_dir.glob("*.json"):
            try:
                # 检查文件修改时间
                if task_file.stat().st_mtime < cutoff_time:
                    task_file.unlink()
                    
                    # 同时删除对应的日志文件
                    log_file = task_file.with_suffix('.jsonl')
                    if log_file.exists():
                        log_file.unlink()
                    
                    cleaned += 1
            except Exception:
                continue
        
        return cleaned


# 全局进度跟踪器实例
_progress_tracker: Optional[ProgressTracker] = None


def get_progress_tracker() -> ProgressTracker:
    """获取进度跟踪器实例（单例）"""
    global _progress_tracker
    if _progress_tracker is None:
        nas_path = os.environ.get(
            "NAS_STORAGE_PATH",
            "/app/mcp-servers/mcp-servers/html_agent"
        )
        _progress_tracker = ProgressTracker(nas_path)
    return _progress_tracker


# 便捷函数
def track_task(task_type: str, description: str):
    """
    任务跟踪装饰器
    
    使用方式：
    @track_task("plan_site", "生成网站计划")
    async def plan_site(description: str):
        ...
    """
    def decorator(func):
        async def wrapper(*args, **kwargs):
            tracker = get_progress_tracker()
            task_id = tracker.create_task(task_type, description)
            
            try:
                # 标记任务开始
                tracker.update_progress(task_id, 0, "running", "任务开始执行")
                
                # 执行任务
                result = await func(*args, **kwargs)
                
                # 标记任务完成
                tracker.update_progress(task_id, 100, "completed", "任务执行成功")
                
                # 在结果中添加任务 ID
                if isinstance(result, dict):
                    result["task_id"] = task_id
                
                return result
                
            except Exception as e:
                # 标记任务失败
                tracker.update_progress(
                    task_id, 
                    tracker.get_task_progress(task_id).get("progress", 0),
                    "failed", 
                    f"任务执行失败: {str(e)}"
                )
                raise
        
        return wrapper
    return decorator


"""AI 内容生成辅助模块 - 为页面生成个性化内容"""
from __future__ import annotations

import json
from typing import Dict, Any, Optional


class AIContentGenerator:
    """AI 内容生成器 - 为各种页面生成个性化内容"""
    
    def __init__(self, agent_instance):
        """初始化生成器
        
        Args:
            agent_instance: SmartWebAgent 实例
        """
        self.agent = agent_instance
    
    def generate_menu_content(
        self, 
        project_name: str, 
        context: Dict[str, Any]
    ) -> Dict[str, Any]:
        """为菜单页面生成个性化内容"""
        
        prompt = f"""你是一个专业的网页内容创作者。
为"{project_name}"生成一个详细的菜单内容。

项目信息：
- 名称：{project_name}
- 描述：{context.get('description', '')}
- 特色：{context.get('features', '')}
- 位置：{context.get('location', '')}
- 目标客户：{context.get('target_audience', '')}

请生成一个完整的菜单，包含：
1. 2-4个类别（如：热饮、冷饮、轻食、甜点）
2. 每个类别3-5个项目
3. 每个项目包含：
   - name: 产品名称（有创意且符合品牌）
   - price: 价格（符合定位，使用"¥"符号）
   - description: 描述（简短吸引人，20字以内）
   - image_topic: 图片主题描述（英文，用于生成图片）

返回 JSON 格式：
{{
  "categories": [
    {{
      "name": "类别名称",
      "items": [
        {{
          "name": "产品名称",
          "price": "¥ XX",
          "description": "产品描述",
          "image_topic": "image description"
        }}
      ]
    }}
  ],
  "description": "菜单页面的副标题描述"
}}

重要：
- 内容要符合项目的定位和风格
- 价格要合理且有梯度
- 描述要诱人且真实
- 直接返回 JSON，不要其他内容
"""
        
        try:
            response = self.agent.client.chat.completions.create(
                model=self.agent.model,
                messages=[
                    {"role": "system", "content": "你是一个专业的内容创作者，擅长为各类商业项目创作个性化内容。"},
                    {"role": "user", "content": prompt}
                ],
                temperature=0.7,
                max_tokens=2000
            )
            
            content = response.choices[0].message.content.strip()
            # 清理可能的 markdown 代码块标记
            if content.startswith("```"):
                lines = content.split('\n')
                content = '\n'.join(lines[1:-1] if lines[-1] == "```" else lines[1:])
            
            return json.loads(content)
            
        except Exception as e:
            print(f"AI 生成菜单内容失败：{e}")
            # 返回空内容，让页面使用默认值
            return {}
    
    def generate_about_content(
        self, 
        project_name: str, 
        context: Dict[str, Any]
    ) -> Dict[str, Any]:
        """为关于页面生成个性化内容"""
        
        prompt = f"""为"{project_name}"生成"关于我们"页面的内容。

项目信息：
- 名称：{project_name}
- 描述：{context.get('description', '')}
- 使命：{context.get('mission', '')}
- 特色：{context.get('features', '')}
- 创始年份：{context.get('year_founded', '2020')}

请生成以下内容（JSON格式）：
{{
  "headline": "页面主标题",
  "subtitle": "副标题（一句话介绍）",
  "story": {{
    "title": "品牌故事标题",
    "content": "品牌故事内容（100-150字）",
    "image_topic": "story image description"
  }},
  "values": ["价值观1", "价值观2", "价值观3", "价值观4"],
  "team": [
    {{
      "name": "团队成员姓名",
      "role": "职位",
      "description": "简介（20字以内）",
      "image_topic": "team member photo description"
    }}
  ],
  "achievements": [
    {{"value": "数值", "label": "标签"}},
    {{"value": "数值", "label": "标签"}}
  ],
  "cta_title": "行动号召标题",
  "cta_text": "行动号召文字"
}}

要求：
- 内容真实可信，避免夸大
- 团队成员生成2-3个核心岗位即可
- 成就数据要合理
- 符合品牌调性
"""
        
        try:
            response = self.agent.client.chat.completions.create(
                model=self.agent.model,
                messages=[
                    {"role": "user", "content": prompt}
                ],
                temperature=0.7,
                max_tokens=1500
            )
            
            content = response.choices[0].message.content.strip()
            if content.startswith("```"):
                lines = content.split('\n')
                content = '\n'.join(lines[1:-1] if lines[-1] == "```" else lines[1:])
            
            return json.loads(content)
            
        except Exception as e:
            print(f"AI 生成关于内容失败：{e}")
            return {}
    
    def generate_contact_content(
        self, 
        project_name: str, 
        context: Dict[str, Any]
    ) -> Dict[str, Any]:
        """为联系页面生成个性化文案"""
        
        prompt = f"""为"{project_name}"生成联系页面的文案。

项目信息：
- 名称：{project_name}
- 类型：{context.get('type', '咖啡店')}
- 特色：{context.get('features', '')}

生成以下文案（JSON格式）：
{{
  "headline": "联系页面标题",
  "subtitle": "副标题（欢迎语）",
  "form_title": "表单标题",
  "form_text": "表单说明文字",
  "response_time": "响应时间承诺"
}}

要求文案：
- 友好亲切
- 专业可信
- 简洁明了
"""
        
        try:
            response = self.agent.client.chat.completions.create(
                model=self.agent.model,
                messages=[
                    {"role": "user", "content": prompt}
                ],
                temperature=0.6,
                max_tokens=500
            )
            
            content = response.choices[0].message.content.strip()
            if content.startswith("```"):
                lines = content.split('\n')
                content = '\n'.join(lines[1:-1] if lines[-1] == "```" else lines[1:])
            
            return json.loads(content)
            
        except Exception as e:
            print(f"AI 生成联系内容失败：{e}")
            return {}
    
    def enhance_project_context(
        self, 
        project_name: str, 
        description: str
    ) -> Dict[str, Any]:
        """增强项目上下文信息，补充缺失的细节"""
        
        prompt = f"""分析项目"{project_name}"并补充必要的商业信息。

项目描述：{description}

请推断并生成以下信息（JSON格式）：
{{
  "type": "项目类型（如：咖啡店、餐厅、科技公司等）",
  "target_audience": "目标客户描述",
  "features": "核心特色（3-5个关键词）",
  "mission": "使命宣言（一句话）",
  "address": "合理的示例地址",
  "phone": "示例电话（使用 010-XXXX-XXXX 格式）",
  "email": "示例邮箱",
  "business_hours": "营业时间"
}}
"""
        
        try:
            response = self.agent.client.chat.completions.create(
                model=self.agent.model,
                messages=[
                    {"role": "user", "content": prompt}
                ],
                temperature=0.5,
                max_tokens=500
            )
            
            content = response.choices[0].message.content.strip()
            if content.startswith("```"):
                lines = content.split('\n')
                content = '\n'.join(lines[1:-1] if lines[-1] == "```" else lines[1:])
            
            return json.loads(content)
            
        except Exception as e:
            print(f"增强上下文失败：{e}")
            return {
                "type": "商业",
                "target_audience": "大众客户",
                "features": "品质、服务、创新",
                "mission": f"为客户提供优质的产品与服务",
                "address": "北京市朝阳区示例路123号",
                "phone": "010-8888-8888",
                "email": f"info@{project_name.lower().replace(' ', '')}.com",
                "business_hours": "周一至周五 9:00-18:00"
            }


def integrate_ai_generation(agent_instance, tool_name: str, params: Dict[str, Any]) -> Dict[str, Any]:
    """集成 AI 生成到工具调用中
    
    Args:
        agent_instance: SmartWebAgent 实例
        tool_name: 工具名称
        params: 工具参数
    
    Returns:
        增强后的参数，包含 AI 生成的内容
    """
    
    # 只处理特定的页面生成工具
    if tool_name not in ["create_menu_page", "create_about_page", "create_contact_page"]:
        return params
    
    # 提取项目信息
    project_name = params.get('project_name', '')
    file_path = params.get('file_path', '')
    
    # 获取或创建上下文
    context = params.get('context', {})
    if not context:
        # 尝试从 agent 的执行历史中提取上下文
        context = {
            'description': agent_instance.latest_user_request or '',
            'project_directory': agent_instance.project_directory
        }
    
    # 创建 AI 内容生成器
    generator = AIContentGenerator(agent_instance)
    
    # 根据工具类型生成相应内容
    ai_content = {}
    if tool_name == "create_menu_page":
        ai_content = generator.generate_menu_content(project_name, context)
    elif tool_name == "create_about_page":
        ai_content = generator.generate_about_content(project_name, context)
    elif tool_name == "create_contact_page":
        ai_content = generator.generate_contact_content(project_name, context)
    
    # 如果需要，增强上下文信息
    if not context.get('address') or not context.get('phone'):
        enhanced_context = generator.enhance_project_context(
            project_name,
            context.get('description', '')
        )
        context.update(enhanced_context)
    
    # 更新参数
    params['context'] = context
    params['ai_content'] = ai_content
    
    return params


__all__ = [
    'AIContentGenerator',
    'integrate_ai_generation'
]


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Smart Web Agent MCP 服务

基于 SmartWebAgent 提供网页生成的规划与执行接口，兼容 Model Context Protocol。
"""

from __future__ import annotations

import asyncio
import json
import os
import sys
import time
import traceback
import zipfile
import tempfile
import aiohttp
from typing import Any, Dict, Optional

# 确保项目根目录在模块搜索路径中
CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))
# 当前文件在 src/htmlgen_mcp/ 下，所以需要向上两级到项目根目录
PROJECT_ROOT = os.path.dirname(os.path.dirname(os.path.dirname(CURRENT_DIR)))
if PROJECT_ROOT not in sys.path:
    sys.path.insert(0, PROJECT_ROOT)

from fastmcp import FastMCP  # type: ignore

import uuid
from pathlib import Path

from htmlgen_mcp.agents.smart_web_agent import SmartWebAgent
from htmlgen_mcp.nas_storage import get_nas_storage
from htmlgen_mcp.nas_log_manager import get_nas_log_manager, ensure_job_log, log_progress, query_progress
from htmlgen_mcp.prompt_enhancer import enhance_prompt_for_real_data
from datetime import datetime

# 使用 NAS 作为默认存储路径
NAS_PATH = os.environ.get("NAS_STORAGE_PATH", "/app/mcp-servers/mcp-servers/html_agent")
# 项目根目录：优先使用 WEB_AGENT_PROJECT_ROOT 环境变量，否则使用 NAS_PATH/projects
DEFAULT_PROJECT_ROOT = os.path.abspath(
    os.environ.get("WEB_AGENT_PROJECT_ROOT", f"{NAS_PATH}/projects")
)
# 是否自动生成项目子目录（可通过环境变量控制）
AUTO_CREATE_PROJECT_DIR = os.environ.get("AUTO_CREATE_PROJECT_DIR", "true").lower() == "true"
# 默认的文件上传URL（固定值）
DEFAULT_UPLOAD_URL = "https://www.mcpcn.cc/api/fileUploadAndDownload/uploadMcpFile"
DEFAULT_MODEL = os.environ.get("WEB_AGENT_MODEL", "qwen3-coder-plus-2025-09-23")
DEFAULT_BASE_URL = os.environ.get(
    "OPENAI_BASE_URL", "https://dashscope.aliyuncs.com/compatible-mode/v1"
)

mcp = FastMCP("smart-web-agent")

# MCP 服务持久化目录：使用 NAS 路径以便集群共享
MCP_SERVICE_NAME = os.environ.get("MCP_SERVICE_NAME", "make_web")
MCP_DATA_ROOT = Path(
    os.environ.get("MCP_DATA_DIR", f"{NAS_PATH}/mcp_data/{MCP_SERVICE_NAME}")
)
MCP_DATA_ROOT.mkdir(parents=True, exist_ok=True)

# 简单的缓存：记录最近一次生成的计划，避免“create_simple_site → execute_plan”时需手动传递
PLAN_CACHE_DIR = MCP_DATA_ROOT / "plan_cache"
PLAN_CACHE_DIR.mkdir(exist_ok=True)

# 进度日志目录，存储每个任务的实时进度
PROGRESS_LOG_DIR = MCP_DATA_ROOT / "progress_logs"
PROGRESS_LOG_DIR.mkdir(exist_ok=True)

# 任务状态目录，每个任务一个 JSON 文件
JOB_STATE_DIR = MCP_DATA_ROOT / "jobs" / "state"
JOB_STATE_DIR.mkdir(parents=True, exist_ok=True)

# 上下文缓存目录
CONTEXT_CACHE_DIR = MCP_DATA_ROOT / "context_cache"
CONTEXT_CACHE_DIR.mkdir(exist_ok=True)

_PLAN_CACHE: dict[tuple[str, str], Dict[str, Any]] = {}
_PLAN_CACHE_BY_ID: dict[str, Dict[str, Any]] = {}
_PROGRESS_LOG_BY_ID: dict[str, str] = {}
_PROGRESS_LOG_BY_JOB: dict[str, str] = {}
_JOB_REGISTRY: dict[str, Dict[str, Any]] = {}
_CONTEXT_CACHE_BY_ID: dict[str, Dict[str, Any]] = {}


def _job_state_path(job_id: str) -> Path:
    return JOB_STATE_DIR / f"{job_id}.json"


def _persist_job_state(job_id: str) -> None:
    job = _JOB_REGISTRY.get(job_id)
    if not job:
        return
    job_copy = {k: v for k, v in job.items() if k not in {"agent"}}
    job_copy["updated_at"] = time.time()
    
    # 同时保存到本地和 NAS
    path = _job_state_path(job_id)
    try:
        path.write_text(
            json.dumps(job_copy, ensure_ascii=False, indent=2), encoding="utf-8"
        )
    except Exception:
        pass
    
    # 保存到 NAS 日志
    try:
        log_manager = get_nas_log_manager()
        plan_id = job.get("plan_id")
        log_manager.create_job_log(job_id, plan_id)
        log_progress(job_id, status="registered", job_info=job_copy)
    except Exception:
        pass


def _load_job_states() -> None:
    for path in JOB_STATE_DIR.glob("*.json"):
        try:
            data = json.loads(path.read_text(encoding="utf-8"))
            job_id = data.get("job_id") or path.stem
            if not job_id:
                continue
            if data.get("status") == "running":
                data["status"] = "stopped"
                data["message"] = "任务在服务器重启时中断，请重新执行"
            _JOB_REGISTRY[job_id] = data
            progress_log = data.get("progress_log")
            if progress_log:
                if not os.path.isabs(progress_log):
                    progress_log = os.path.join(PROJECT_ROOT, progress_log)
                _PROGRESS_LOG_BY_JOB[job_id] = progress_log
                plan_id = data.get("plan_id")
                if plan_id and plan_id not in _PROGRESS_LOG_BY_ID:
                    _PROGRESS_LOG_BY_ID[plan_id] = progress_log
        except Exception:
            continue


_load_job_states()


def _context_cache_path(context_id: str) -> Path:
    return CONTEXT_CACHE_DIR / f"{context_id}.json"


def _load_context_cache() -> None:
    for path in CONTEXT_CACHE_DIR.glob("*.json"):
        try:
            data = json.loads(path.read_text(encoding="utf-8"))
            context_id = data.get("context_id") or path.stem
            if not context_id:
                continue
            data.setdefault("path", str(path))
            _CONTEXT_CACHE_BY_ID[context_id] = data
        except Exception:
            continue


_load_context_cache()


def _resolve_cached_context(context_id: Optional[str]) -> Optional[Dict[str, Any]]:
    if not context_id:
        return None
    cached = _CONTEXT_CACHE_BY_ID.get(context_id)
    if cached:
        return cached
    path = _context_cache_path(context_id)
    if path.exists():
        try:
            data = json.loads(path.read_text(encoding="utf-8"))
            data.setdefault("path", str(path))
            _CONTEXT_CACHE_BY_ID[context_id] = data
            return data
        except Exception:
            return None
    return None


def _resolve_project_directory(project_root: Optional[str], project_name: Optional[str] = None) -> str:
    """
    解析项目目录路径
    
    Args:
        project_root: 项目根目录或完整路径
        project_name: 项目名称（可选）
    
    Returns:
        完整的项目目录路径
    """
    if project_root:
        # 如果提供了 project_root
        if os.path.isabs(project_root):
            # 绝对路径：直接使用
            abs_path = project_root
        else:
            # 相对路径：相对于默认根目录
            # 如果 project_root 看起来像项目名（不含/），则作为子目录
            if '/' not in project_root and '\\' not in project_root:
                abs_path = os.path.join(DEFAULT_PROJECT_ROOT, project_root)
            else:
                # 包含路径分隔符，作为相对路径处理
                abs_path = os.path.abspath(os.path.join(DEFAULT_PROJECT_ROOT, project_root))
    else:
        # 没有提供 project_root，使用默认根目录
        base = DEFAULT_PROJECT_ROOT
        
        # 如果提供了 project_name 且启用了自动创建子目录
        if project_name and AUTO_CREATE_PROJECT_DIR:
            # 清理项目名称，去除特殊字符
            safe_name = "".join(c for c in project_name if c.isalnum() or c in (' ', '-', '_', '.'))
            safe_name = safe_name.strip().replace(' ', '_')
            if safe_name:
                abs_path = os.path.join(base, safe_name)
            else:
                abs_path = base
        else:
            abs_path = base
    
    # 创建目录
    os.makedirs(abs_path, exist_ok=True)
    return abs_path


def _build_agent(
    project_directory: str,
    model: Optional[str] = None,
    *,
    show_code: bool = False,
    verbose: bool = False,
    save_output: bool = False,
    force_single_page: bool = True,
) -> SmartWebAgent:
    return SmartWebAgent(
        project_directory=project_directory,
        model=model or DEFAULT_MODEL,
        show_code=show_code,
        verbose=verbose,
        save_output=save_output,
        force_single_page=force_single_page,
    )


def _prepare_agent_run(agent: SmartWebAgent, description: str) -> None:
    agent.execution_start_time = time.time()
    agent.execution_history = []
    agent.created_files = []
    agent.latest_user_request = description
    agent.current_plan = None


def _decode_plan(agent: SmartWebAgent, plan: Any) -> Dict[str, Any]:
    if isinstance(plan, str):
        plan = json.loads(plan)
    if not isinstance(plan, dict):
        raise ValueError("plan 应该是 JSON 对象")
    source_description = plan.pop("__source_description", None)
    plan.pop("__plan_id", None)
    plan.pop("__plan_path", None)
    if source_description:
        agent.latest_user_request = source_description
    repaired = agent._repair_plan_tools_sequence(plan)
    if not agent._validate_plan(repaired):
        raise ValueError("执行计划不完整或格式错误")
    agent.current_plan = repaired
    return repaired


def _create_plan(agent: SmartWebAgent, description: str) -> Dict[str, Any]:
    _prepare_agent_run(agent, description)
    enhanced = agent._enhance_user_input(description)
    plan = agent._get_execution_plan_with_retry(enhanced)
    if not plan:
        raise RuntimeError("未能生成执行计划，请检查描述或模型配置")
    if not isinstance(plan, dict):
        raise ValueError("模型返回的计划格式异常，应为 JSON 对象")
    plan_id = uuid.uuid4().hex
    plan_path = PLAN_CACHE_DIR / f"{plan_id}.json"

    plan_for_storage = plan.copy()
    plan_for_storage["__source_description"] = description
    plan_path.write_text(
        json.dumps(plan_for_storage, ensure_ascii=False, indent=2), encoding="utf-8"
    )

    plan["__source_description"] = description

    _PLAN_CACHE_BY_ID[plan_id] = {
        "plan": plan,
        "project_directory": agent.project_directory,
        "description": description,
        "source_description": description,
        "path": str(plan_path),
        "plan_id": plan_id,
    }

    cache_key = (agent.project_directory, description)
    _PLAN_CACHE[cache_key] = {
        "plan": plan,
        "plan_id": plan_id,
        "description": description,
        "source_description": description,
    }

    plan["__plan_id"] = plan_id
    plan["__plan_path"] = str(plan_path)
    return plan


def _execute_plan(
    agent: SmartWebAgent,
    plan: Dict[str, Any],
    *,
    progress_log_path: Optional[str] = None,
    job_id: Optional[str] = None,
) -> Dict[str, Any]:
    progress_events: list[Dict[str, Any]] = []

    def _collect(event: Dict[str, Any]) -> None:
        if isinstance(event, dict):
            progress_events.append(event)
            
            # 写入本地日志
            if progress_log_path:
                try:
                    log_record = dict(event)
                    log_record.setdefault("timestamp", time.time())
                    with open(progress_log_path, "a", encoding="utf-8") as log_file:
                        log_file.write(json.dumps(log_record, ensure_ascii=False))
                        log_file.write("\n")
                except Exception:
                    pass
            
            # 同时写入 NAS 日志
            if job_id:
                try:
                    log_progress(job_id, **event)
                except Exception:
                    pass

    results = agent._execute_plan_with_recovery(
        plan,
        confirm_each_step=False,  # 后台执行模式，不需要确认
        progress_callback=_collect,
    )

    if any(
        r.get("status") == "success"
        and r.get("tool") in {"create_html_file", "create_css_file"}
        for r in results
    ):
        agent._run_consistency_review(plan)

    report = agent._generate_execution_report(plan, results)

    return {
        "report": report,
        "progress": progress_events,
        "results": results,
        "created_files": list(agent.created_files),
    }


# @mcp.tool()
# async def plan_site(
#     description: str,
#     project_root: Optional[str] = None,
#     context_id: Optional[str] = None,
#     context_content: Optional[str] = None,
# ) -> Dict[str, Any]:
#     """根据需求与上下文生成网页构建计划，所有信息都会交由 AI 模型统一分析。
#
#     ⚠️ 重要参数说明：
#     - description: 网站建设需求或目标，侧重描述要实现的结构、功能、风格
#     - context_content: 🔥 核心参数！网页制作所需的全部原始文本或数据
#       * 例如：地图查询结果、咖啡馆信息、产品数据、营业时间、地址等
#       * 这是模型获取上下文内容的唯一入口，不会自动从 description 推断
#       * 请把需要引用的完整信息直接放入该参数
#       * 支持各种格式：文本、JSON字符串、结构化数据等
#
#     其他参数：
#     - project_root: 可选，自定义项目根目录；缺省时使用默认目录
#     - context_id: 可选，引用已缓存的上下文快照以复用历史资料
#
#     返回值说明：
#     - status: 操作状态 ("success" 或 "error")
#     - plan_id: 生成的计划唯一标识符，用于后续执行
#     - plan_path: 计划 JSON 文件的保存路径
#     - project_directory: 解析后的项目目录路径
#     - model: 使用的 AI 模型名称
#
#     💡 使用提示：
#     如果你先用其他工具（如地图查询）获取了数据，请将结果完整传递给 context_content，
#     这样 AI 就能基于真实数据生成个性化网站。
#     """
#     try:
#         project_dir = _resolve_project_directory(project_root)
#         agent = _build_agent(project_dir)
#
#         # 直接使用 description，如果有额外的上下文则附加
#         final_description = description
#
#         # 如果提供了 context_content，附加到描述中
#         if context_content:
#             final_description = f"{description}\n\n【附加内容】\n{context_content}"
#
#         # 如果提供了 context_id，尝试获取缓存的内容
#         elif context_id:
#             cached = _resolve_cached_context(context_id)
#             if cached:
#                 cached_content = cached.get("context")
#                 if cached_content:
#                     # 尝试解析JSON格式的增强数据
#                     try:
#                         enhanced_data = json.loads(cached_content)
#                         if "original_content" in enhanced_data:
#                             cached_content = enhanced_data["original_content"]
#                     except (json.JSONDecodeError, TypeError):
#                         pass  # 使用原始内容
#
#                     if cached_content:
#                         final_description = f"{description}\n\n【缓存内容】\n{cached_content}"
#
#         # 让 AI 模型直接处理所有内容
#         plan = await asyncio.to_thread(_create_plan, agent, final_description)
#         plan_id = plan.pop("__plan_id", None)
#         plan_path = plan.pop("__plan_path", None)
#
#         return {
#             "status": "success",
#             "plan_id": plan_id,
#             "plan_path": plan_path,
#             "project_directory": project_dir,
#             "model": agent.model,
#             "message": "计划已生成，AI模型已分析所提供的全部内容"
#         }
#     except Exception as exc:
#         return {
#             "status": "error",
#             "message": str(exc),
#             "traceback": traceback.format_exc(),
#         }


@mcp.tool()
async def execute_plan(
    plan_id: str,
    project_root: Optional[str] = None,
    # auto_plan: bool = False,  # 已禁用，没有实际作用
    # confirm_each_step: bool = False,  # 后台执行模式下用户无法交互确认
    # show_code: bool = False,  # 后台执行时用户看不到输出
    # verbose: bool = False,  # 后台执行时详细日志意义不大
    save_output: bool = False,
    progress_log: Optional[str] = None,
    auto_upload: bool = False,
    upload_url: Optional[str] = None,
) -> Dict[str, Any]:
    """执行网页构建计划，始终以后台模式运行。

    参数详细说明：
    - plan_id: 计划的唯一标识符
        由 plan_site 工具返回的计划ID，用于从缓存或文件系统中查找对应的执行计划。
        例如："a1b2c3d4e5f6..." 这样的32位十六进制字符串。

    - project_root: 网站文件生成的目标目录路径
        指定项目文件的输出位置，可以是绝对路径或相对路径。
        例如："/path/to/my/website" 或 "./my-project"
        如果目录不存在，系统会自动创建。

    - save_output: 是否保存执行过程的详细输出到文件
        True: 会自动创建进度日志文件，记录所有执行步骤和结果
        False: 不创建进度日志文件，只保留基本的任务状态信息
        建议在调试或需要详细追踪时设置为 True

    - progress_log: 自定义进度日志文件的保存路径（可选）
        如果指定：使用该路径保存进度日志（JSONL格式）
        如果未指定但 save_output=True：自动在 ~/.mcp/make_web/progress_logs 目录创建时间戳命名的日志文件（可通过环境变量覆盖）
        如果都未指定：不记录进度日志
        路径可以是绝对路径或相对于 project_root 的相对路径

    - auto_upload: 是否在构建完成后自动上传到MCP服务器
        True: 构建完成后自动打包为ZIP并上传，返回访问URL
        False: 仅构建，不上传
        默认为 False

    - upload_url: 文件上传API地址（可选）
        默认为 "https://www.mcpcn.cc/api/fileUploadAndDownload/uploadMcpFile"
        只在 auto_upload=True 时生效，一般不需要修改

    执行流程：
    - 任务始终在后台异步执行，立即返回 job_id 和 progress_log 路径
    - 使用 get_progress(job_id=..., limit=...) 可以实时查询执行状态和进度
    - 支持通过进度日志文件追踪详细的执行步骤和结果
    - 执行完成后可以获取完整的执行报告和生成的文件列表
    - 🎯 新增：auto_upload=True时，构建完成(100%)后自动上传并返回访问URL
    """
    try:
        # 如果没有提供project_root，尝试从缓存中获取
        if not project_root:
            cached_by_id = _PLAN_CACHE_BY_ID.get(plan_id)
            if cached_by_id and cached_by_id.get("project_directory"):
                project_root = cached_by_id["project_directory"]
            else:
                # 尝试从文件中读取
                possible_paths = [
                    PLAN_CACHE_DIR / f"{plan_id}.json",
                    PLAN_CACHE_DIR / f"simple_site_plan_{plan_id}.json",
                ]
                for path in possible_paths:
                    if path.exists():
                        try:
                            plan_data = json.loads(path.read_text(encoding="utf-8"))
                            project_root = plan_data.get("project_directory")
                            if project_root:
                                break
                        except Exception:
                            pass
                
                if not project_root:
                    # 使用默认根目录，但不添加子目录
                    project_root = None
        
        # 从计划中获取项目名称（如果有）
        project_name = None
        if plan_id:
            cached_plan = _PLAN_CACHE_BY_ID.get(plan_id)
            if cached_plan:
                # 尝试从缓存中获取项目名称
                project_name = cached_plan.get("site_title") or cached_plan.get("project_name")
        
        project_dir = _resolve_project_directory(project_root, project_name)

        # 移除 auto_plan 检查，因为参数已被移除

        if progress_log:
            progress_log_path = (
                progress_log
                if os.path.isabs(progress_log)
                else os.path.join(project_dir, progress_log)
            )
        elif save_output:
            progress_log_path = os.path.join(
                PROGRESS_LOG_DIR, f"agent_progress_{int(time.time())}.jsonl"
            )
        else:
            progress_log_path = None

        if progress_log_path:
            try:
                Path(progress_log_path).parent.mkdir(parents=True, exist_ok=True)
                Path(progress_log_path).write_text("", encoding="utf-8")
            except Exception:
                progress_log_path = None

        agent = _build_agent(
            project_dir,
            save_output=save_output,
        )

        # 通过 plan_id 查询计划
        cached_by_id = _PLAN_CACHE_BY_ID.get(plan_id)

        # 尝试多种文件命名格式
        possible_paths = [
            PLAN_CACHE_DIR / f"{plan_id}.json",  # 标准格式
            PLAN_CACHE_DIR
            / f"simple_site_plan_{plan_id}.json",  # create_simple_site格式
        ]

        plan_path = None
        for path in possible_paths:
            if path.exists():
                plan_path = path
                break

        if not cached_by_id and plan_path:
            try:
                cached_plan_file = json.loads(plan_path.read_text(encoding="utf-8"))
                source_description = None
                if isinstance(cached_plan_file, dict):
                    source_description = cached_plan_file.get("__source_description")
                    # 提取实际的plan部分，而不是整个文件内容
                    actual_plan = cached_plan_file.get("plan")
                    if not actual_plan:
                        # 如果没有plan字段，可能是旧格式，直接使用文件内容
                        actual_plan = cached_plan_file

                cached_by_id = {
                    "plan": actual_plan,  # 传递实际的plan内容
                    "project_directory": project_dir,
                    "plan_id": plan_id,
                    "description": source_description
                    or cached_plan_file.get("description"),
                    "source_description": source_description,
                }
                _PLAN_CACHE_BY_ID[plan_id] = cached_by_id
            except Exception:
                cached_by_id = None

        if not cached_by_id:
            raise ValueError(
                f"未找到 plan_id '{plan_id}' 对应的计划，请先调用 create_simple_site 生成计划"
            )

        plan_dict = _decode_plan(agent, cached_by_id.get("plan"))
        effective_description = (
            cached_by_id.get("source_description")
            or cached_by_id.get("description")
            or plan_dict.get("task_analysis")
            or plan_dict.get("project_name")
            or plan_dict.get("site_type")
            or "Web Project Execution"
        )

        _prepare_agent_run(agent, effective_description)
        agent.current_plan = plan_dict

        # 始终以后台模式执行
        job_id = uuid.uuid4().hex
        job_info = {
            "job_id": job_id,
            "status": "running",
            "plan_id": plan_id,
            "description": effective_description,
            "project_directory": project_dir,
            "model": agent.model,
            "progress_log": progress_log_path,
            "started_at": time.time(),
            "updated_at": time.time(),
        }
        _JOB_REGISTRY[job_id] = job_info

        if plan_id and progress_log_path:
            _PROGRESS_LOG_BY_ID[plan_id] = progress_log_path
        if progress_log_path:
            _PROGRESS_LOG_BY_JOB[job_id] = progress_log_path

        _persist_job_state(job_id)

        # 使用默认URL如果没有提供
        effective_upload_url = upload_url or DEFAULT_UPLOAD_URL
        
        asyncio.create_task(
            _run_execution_job(
                job_id,
                agent,
                plan_dict,
                progress_log_path=progress_log_path,
                auto_upload=auto_upload,
                upload_url=effective_upload_url,
            )
        )

        if auto_upload:
            message = (
                "执行已在后台启动（含自动上传）：调用 get_progress(job_id='{}', limit=20) "
                "可获取构建和上传进度。完成后将返回网站访问URL。"
            ).format(job_id)
        else:
            message = (
                "执行已在后台启动：调用 get_progress(job_id='{}', limit=20) "
                "或传入 progress_log='{}' 可获取实时进度"
            ).format(job_id, progress_log_path or "<未启用进度日志>")

        return {
            "status": "started",
            "job_id": job_id,
            "plan_id": plan_id,
            "progress_log": progress_log_path,
            "auto_upload": auto_upload,
            "upload_url": upload_url if auto_upload else None,
            "message": message,
        }
    except Exception as exc:
        return {
            "status": "error",
            "message": str(exc),
            "traceback": traceback.format_exc(),
        }


async def _run_execution_job(
    job_id: str,
    agent: SmartWebAgent,
    plan_dict: Dict[str, Any],
    *,
    progress_log_path: Optional[str],
    auto_upload: bool = False,
    upload_url: str = None,
) -> None:
    job_info = _JOB_REGISTRY.get(job_id)
    if not job_info:
        return

    try:
        result = await asyncio.to_thread(
            _execute_plan,
            agent,
            plan_dict,
            progress_log_path=progress_log_path,
        )
        job_info["status"] = "completed"
        job_info["result"] = result
        job_info["completed_at"] = time.time()
        _persist_job_state(job_id)

        # 🎯 关键：任务完成后自动上传
        if auto_upload and job_info.get("project_directory"):
            try:
                # 记录上传开始
                job_info["upload_status"] = "uploading"
                _persist_job_state(job_id)

                # 只在不是默认值时传递upload_url
                upload_params = {"folder_path": job_info["project_directory"]}
                if upload_url and upload_url != DEFAULT_UPLOAD_URL:
                    upload_params["upload_url"] = upload_url
                
                upload_result = await upload_project_to_mcp_server(**upload_params)

                # 记录上传结果
                job_info["upload_result"] = upload_result
                job_info["upload_status"] = upload_result["status"]
                if upload_result["status"] == "success":
                    job_info["website_url"] = upload_result["url"]
                    job_info["upload_completed_at"] = time.time()

                    # 添加到进度日志
                    if progress_log_path:
                        upload_event = {
                            "timestamp": time.time(),
                            "type": "upload_completed",
                            "status": "success",
                            "website_url": upload_result["url"],
                            "message": upload_result["message"],
                        }
                        try:
                            with open(
                                progress_log_path, "a", encoding="utf-8"
                            ) as log_file:
                                log_file.write(
                                    json.dumps(upload_event, ensure_ascii=False)
                                )
                                log_file.write("\n")
                        except Exception:
                            pass

                _persist_job_state(job_id)
            except Exception as upload_exc:
                job_info["upload_status"] = "failed"
                job_info["upload_error"] = str(upload_exc)
                _persist_job_state(job_id)

    except Exception as exc:
        job_info["status"] = "failed"
        job_info["error"] = {
            "message": str(exc),
            "traceback": traceback.format_exc(),
        }
        _persist_job_state(job_id)
    finally:
        job_info["updated_at"] = time.time()
        _persist_job_state(job_id)


@mcp.tool()
async def create_simple_site(
    description: str,
    site_title: str = "我的网站",
    project_root: Optional[str] = None,
    model: Optional[str] = None,
    context_id: Optional[str] = None,
    context_content: Optional[str] = None,
) -> Dict[str, Any]:
    """使用AI分析需求，生成简单但美观的网站计划。

    ⚠️ 重要参数说明：
    - description: 网站需求描述，例如"个人作品展示网站"、"小餐厅官网"、"博客网站"等
    - site_title: 网站标题
    - context_content: 🔥 核心参数！用于传递网页制作所需的所有原始数据内容
      * 例如：咖啡馆列表、产品介绍、菜单内容、地址信息、营业时间等
      * 这是AI获取具体业务信息的唯一渠道，请务必将查询到的详细信息完整传入
      * 如果有地图查询结果、API返回数据等，都应该放在这个参数中
      * 格式可以是文本、JSON字符串或结构化数据的字符串表示

    其他参数：
    - project_root: 项目根目录，缺省使用默认目录
    - model: 使用的AI模型，缺省使用默认模型
    - context_id: 可选，引用已缓存的上下文快照以复用历史资料

    图片配置参数（可选）：
    - image_style: 图片风格 (professional|artistic|minimal|vibrant|luxury)
    - image_topics: 用户自定义的图片主题列表，如 ["modern office", "team collaboration"]
    - include_gallery: 是否在网站中包含图片画廊功能
    - image_provider: 图片提供商 (pollinations|dicebear|robohash)

    返回值说明：
    - status: 操作状态 ("success" 或 "error")
    - plan_id: 生成的计划唯一标识符，用于后续执行
    - plan_path: 计划 JSON 文件的保存路径
    - project_directory: 解析后的项目目录路径
    - plan: 生成的简化执行计划概览
    - context_id: 上下文缓存ID（如果使用了上下文）

    使用流程：
    1. 调用此工具生成计划，获得 plan_id
    2. 使用 plan_id 调用 execute_plan 执行构建
    3. 使用 plan_id 调用 get_progress 查询进度

    💡 使用提示：
    如果你有地图查询结果、API数据等，请将完整信息传递给 context_content 参数，
    这样AI就能基于真实数据来生成个性化的网站内容。
    """
    try:
        # 使用指定模型或默认模型
        used_model = model or DEFAULT_MODEL

        # 使用新的路径解析逻辑
        # 如果没有提供 project_root，使用 site_title 作为项目名
        project_directory = _resolve_project_directory(project_root, site_title)

        # 处理上下文
        context_data = ""
        actual_context_id = context_id

        if context_id and context_id in _CONTEXT_CACHE_BY_ID:
            # 使用缓存的上下文
            cached_context = _CONTEXT_CACHE_BY_ID[context_id]
            context_data = cached_context.get("content", "")
        elif context_content:
            # 使用新提供的上下文内容
            context_data = context_content
            # 生成新的上下文ID并缓存
            actual_context_id = str(uuid.uuid4())
            _CONTEXT_CACHE_BY_ID[actual_context_id] = {
                "content": context_content,
                "created_at": time.time(),
                "site_title": site_title,
                "description": description,
            }

        # 创建AI代理进行分析
        agent = SmartWebAgent(
            project_directory=project_directory,
            model=used_model,
            show_code=False,
            verbose=False,
            force_single_page=True,
        )

        # 如果有上下文内容，将其整合到描述中
        enhanced_description = description
        if context_data:
            enhanced_description = f"""{description}

【必须使用的具体数据内容】：
{context_data}

【重要提示】：上述数据是真实的业务数据，必须完整准确地展示在网页中，不要生成虚构的示例内容。"""

        # 构建改进的提示词，强调使用真实数据
        simple_prompt = f"""请为以下需求创建一个网站，并严格使用提供的真实数据：

**网站标题**: {site_title}
**具体需求和数据**: 
{enhanced_description}

**执行要求**：
1. 【数据要求】如果提供了具体数据（如店铺列表、产品信息等），必须100%使用这些真实数据，不要创建虚构内容
2. 【内容展示】将所有提供的数据项完整展示，使用合适的布局（如卡片、列表、表格等）
3. 【样式设计】保持简洁美观，使用响应式设计
4. 【代码限制】CSS不超过300行，避免复杂特效
5. 【功能实现】包含基础交互功能（导航、滚动等）

**特别强调**：
- 当创建HTML内容时，必须使用上面提供的真实数据
- 不要生成"示例客户评价"、"虚拟定价方案"等占位内容
- 如果是咖啡馆列表，就展示真实的咖啡馆名称和地址
- 如果是产品信息，就展示真实的产品数据
- 每个create_html_file或add_content_section工具调用时，都要包含真实数据

请生成3-6个步骤的执行计划，确保每个步骤都能正确使用提供的数据。
"""

        # 生成简化计划（仅规划，不执行）
        # 传递强化后的提示词，确保AI使用真实数据
        # 使用提示词增强器进一步强化
        final_prompt = enhance_prompt_for_real_data(simple_prompt, context_data)
        plan = agent._get_execution_plan(final_prompt)

        # 在计划中标记为简单网站类型和相关信息
        plan["site_type"] = "simple"
        plan["complexity"] = "简单但美观"
        plan["css_limit"] = "不超过300行"
        plan["model_used"] = used_model
        plan["has_context"] = bool(context_data)
        if actual_context_id:
            plan["context_id"] = actual_context_id

        # 生成唯一的计划ID
        plan_id = str(uuid.uuid4())

        # 构建完整的源描述（包含上下文）
        # 使用enhanced_description以确保数据被传递
        source_description = enhanced_description

        # 在计划中添加源描述字段
        plan["__source_description"] = source_description
        plan["__plan_id"] = plan_id

        # 保存计划到缓存（结构与 create_simple_site 保持一致，便于 execute_plan 复用逻辑）
        cached_entry = {
            "plan": plan,
            "project_directory": project_directory,
            "description": description,
            "source_description": source_description,
            "site_title": site_title,
            "plan_id": plan_id,
        }

        _PLAN_CACHE_BY_ID[plan_id] = cached_entry
        cache_key = (project_directory, description)
        _PLAN_CACHE[cache_key] = cached_entry

        # 将上下文信息关联到计划
        if actual_context_id:
            _PROGRESS_LOG_BY_ID[plan_id] = actual_context_id

        # 保存计划到文件
        plan_filename = f"simple_site_plan_{plan_id}.json"
        plan_path = PLAN_CACHE_DIR / plan_filename

        try:
            # 构建完整的源描述（包含上下文）
            source_description = description
            if context_data:
                source_description = f"{description}\n\n【附加内容】\n{context_data}"

            plan_data = {
                "plan_id": plan_id,
                "site_title": site_title,
                "description": description,
                "project_directory": project_directory,
                "model": used_model,
                "plan_type": "simple_site",
                "created_at": time.time(),
                "plan": plan,
                "__source_description": source_description,  # 添加完整的源描述字段
            }
            if actual_context_id:
                plan_data["context_id"] = actual_context_id
                plan_data["has_context"] = True

            with open(plan_path, "w", encoding="utf-8") as f:
                json.dump(plan_data, f, ensure_ascii=False, indent=2)
        except Exception:
            # 文件保存失败不影响主流程
            pass

        # 生成计划概览
        tools_sequence = plan.get("tools_sequence", [])
        plan_overview = {
            "description": plan.get("description", "简单网站构建计划"),
            "steps": len(tools_sequence),
            "step_list": [
                step.get("description", step.get("tool", "")) for step in tools_sequence
            ],
            "estimated_files": plan.get("estimated_files", "3-5个文件"),
            "features": plan.get("features", ["响应式设计", "轻量级样式", "基础交互"]),
            "has_context": bool(context_data),
        }

        result = {
            "status": "success",
            "message": f"简单网站计划生成成功，包含{len(tools_sequence)}个执行步骤",
            "plan_id": plan_id,
            "plan_path": str(plan_path),
            "project_directory": project_directory,
            "plan": plan_overview,
            "model_used": used_model,
            "next_step": f"使用 execute_plan(plan_id='{plan_id}') 开始构建网站",
        }

        if actual_context_id:
            result["context_id"] = actual_context_id
            result["context_used"] = True

        return result

    except Exception as exc:
        return {
            "status": "error",
            "message": str(exc),
            "traceback": traceback.format_exc(),
        }


@mcp.tool()
async def get_progress(
    plan_id: Optional[str] = None,
    job_id: Optional[str] = None,
    log_path: Optional[str] = None,
    limit: int = 20,
) -> Dict[str, Any]:
    """查询网页构建任务的执行进度和状态。

    参数说明：
    - plan_id: create_simple_site 返回的 ID，可定位默认的进度日志。
    - job_id: execute_plan(background=true) 返回的任务 ID，可直接查询后台任务状态。
    - log_path: 进度日志 JSONL 文件的路径（绝对或相对），优先级最高。
    - limit: 返回的最新事件数量（默认 20 条）。

    使用提示：
    1. 推荐直接传入 execute_plan 返回的 job_id 和 progress_log。
    2. 若未提供 log_path，本工具会按 job_id -> plan_id 的顺序尝试查找已缓存的日志。
    3. 返回内容包括最新事件列表、日志路径以及（若有）任务快照或结果摘要，可用于持续追踪构建进度。
    """

    try:
        # SSE 模式优化：使用异步 I/O
        loop = asyncio.get_event_loop()
        
        if limit <= 0:
            limit = 20

        job_info = None
        resolved_path = None

        if job_id:
            job_info = _JOB_REGISTRY.get(job_id)
            if job_info and not plan_id:
                plan_id = job_info.get("plan_id")
            if job_id in _PROGRESS_LOG_BY_JOB:
                resolved_path = _PROGRESS_LOG_BY_JOB[job_id]
            elif job_info and job_info.get("progress_log"):
                resolved_path = job_info.get("progress_log")

        if not resolved_path and plan_id and plan_id in _PROGRESS_LOG_BY_ID:
            resolved_path = _PROGRESS_LOG_BY_ID[plan_id]

        if log_path:
            resolved_path = log_path

        if resolved_path:
            if not os.path.isabs(resolved_path):
                candidate = os.path.join(PROJECT_ROOT, resolved_path)
                # 异步检查文件存在
                exists = await loop.run_in_executor(None, os.path.exists, candidate)
                if exists:
                    resolved_path = candidate
                else:
                    alt = os.path.sep + resolved_path.lstrip(os.path.sep)
                    exists_alt = await loop.run_in_executor(None, os.path.exists, alt)
                    if exists_alt:
                        resolved_path = alt

        # 异步检查最终路径
        path_exists = await loop.run_in_executor(
            None, lambda: resolved_path and os.path.exists(resolved_path)
        )
        
        if not path_exists:
            return {
                "status": "error",
                "message": "未找到进度日志，请确认 job_id/plan_id 或提供 log_path（注意绝对路径需以/开头，扩展名为 .jsonl）",
            }

        # 异步读取文件
        def read_file():
            events = []
            total = 0
            try:
                with open(resolved_path, "r", encoding="utf-8") as f:
                    lines = f.readlines()
                    total = len(lines)
                    for line in lines[-limit:]:
                        line = line.strip()
                        if not line:
                            continue
                        try:
                            events.append(json.loads(line))
                        except Exception:
                            continue
            except Exception:
                pass
            return events, total
        
        events, total_lines = await loop.run_in_executor(None, read_file)

        response: Dict[str, Any] = {
            "status": "success",
            "plan_id": plan_id,
            "job_id": job_id,
            "log_path": resolved_path,
            "events": events,
            "total_records": total_lines,
            "returned": len(events),
        }

        if job_info:
            snapshot_keys = [
                "job_id",
                "status",
                "plan_id",
                "progress_log",
                "started_at",
                "updated_at",
                "completed_at",
                "project_directory",
                "model",
                "upload_status",
                "website_url",
                "upload_completed_at",
            ]
            job_snapshot = {
                k: job_info.get(k) for k in snapshot_keys if job_info.get(k) is not None
            }

            if job_info.get("status") == "completed":
                job_snapshot["result_summary"] = {
                    "report": job_info.get("result", {}).get("report"),
                    "created_files": job_info.get("result", {}).get("created_files"),
                }
                # 添加上传结果信息
                if job_info.get("upload_result"):
                    job_snapshot["upload_result"] = job_info.get("upload_result")

            if job_info.get("status") == "failed":
                job_snapshot["error"] = job_info.get("error")

            # 添加上传错误信息
            if job_info.get("upload_error"):
                job_snapshot["upload_error"] = job_info.get("upload_error")

            response["job"] = job_snapshot

        return response
    except Exception as exc:
        return {
            "status": "error",
            "message": str(exc),
            "traceback": traceback.format_exc(),
        }


@mcp.tool()
async def upload_project_to_mcp_server(
    folder_path: str,
    upload_url: Optional[str] = None,
) -> Dict[str, Any]:
    """将项目文件夹打包成ZIP并上传到MCP服务器。

    参数说明：
    - folder_path: 项目文件夹的绝对路径
    - upload_url: 上传API地址（可选）
        默认为 "https://www.mcpcn.cc/api/fileUploadAndDownload/uploadMcpFile"
        一般不需要修改

    返回值：
    - status: 上传状态 ("success" 或 "error")
    - url: 上传成功后返回的文件访问URL
    - zip_path: 临时ZIP文件路径（用于调试）
    - message: 状态信息
    """
    try:
        # 使用默认URL如果没有提供
        effective_upload_url = upload_url or DEFAULT_UPLOAD_URL
        
        # 验证文件夹路径
        if not os.path.exists(folder_path):
            return {"status": "error", "message": f"项目文件夹不存在: {folder_path}"}

        if not os.path.isdir(folder_path):
            return {"status": "error", "message": f"路径不是文件夹: {folder_path}"}

        # 创建临时ZIP文件
        project_name = os.path.basename(folder_path.rstrip("/"))
        temp_dir = tempfile.gettempdir()
        zip_filename = f"{project_name}_{int(time.time())}.zip"
        zip_path = os.path.join(temp_dir, zip_filename)

        # 打包项目文件
        with zipfile.ZipFile(zip_path, "w", zipfile.ZIP_DEFLATED) as zipf:
            for root, dirs, files in os.walk(folder_path):
                for file in files:
                    file_path = os.path.join(root, file)
                    # 计算相对路径，保持目录结构
                    arcname = os.path.relpath(file_path, folder_path)
                    zipf.write(file_path, arcname)

        # 检查ZIP文件大小
        zip_size = os.path.getsize(zip_path)
        if zip_size > 50 * 1024 * 1024:  # 50MB限制
            os.remove(zip_path)
            return {
                "status": "error",
                "message": f"ZIP文件过大: {zip_size / 1024 / 1024:.1f}MB，超过50MB限制",
            }

        # 上传文件
        async with aiohttp.ClientSession() as session:
            with open(zip_path, "rb") as f:
                data = aiohttp.FormData()
                data.add_field(
                    "file", f, filename=zip_filename, content_type="application/zip"
                )

                async with session.post(effective_upload_url, data=data) as response:
                    response_text = await response.text()

                    if response.status != 200:
                        return {
                            "status": "error",
                            "message": f"上传失败，HTTP {response.status}: {response_text}",
                        }

                    # 解析响应JSON
                    try:
                        result = json.loads(response_text)
                        if result.get("code") == 0 and result.get("data", {}).get(
                            "url"
                        ):
                            # 清理临时文件
                            try:
                                os.remove(zip_path)
                            except:
                                pass

                            return {
                                "status": "success",
                                "url": result["data"]["url"],
                                "message": f"项目 '{project_name}' 上传成功",
                                "zip_size": f"{zip_size / 1024:.1f}KB",
                            }
                        else:
                            return {
                                "status": "error",
                                "message": f"上传失败: {result.get('msg', '未知错误')}",
                                "response": response_text,
                            }
                    except json.JSONDecodeError:
                        return {
                            "status": "error",
                            "message": f"响应解析失败: {response_text}",
                        }

    except Exception as exc:
        # 清理临时文件
        if "zip_path" in locals() and os.path.exists(zip_path):
            try:
                os.remove(zip_path)
            except:
                pass

        return {
            "status": "error",
            "message": str(exc),
            "traceback": traceback.format_exc(),
        }


@mcp.tool()
async def deploy_folder_or_zip(
    folder_path: str, env: str = "Production"
) -> Dict[str, Any]:
    """将构建好的网站文件夹或ZIP文件部署到EdgeOne Pages。

    参数说明：
    - folder_path: 本地文件夹或ZIP文件的绝对路径
        指定要部署的前端构建产物位置，可以是：
        * 构建好的静态网站文件夹（如 ./dist, ./build 等）
        * 包含网站文件的ZIP压缩包
        系统会自动检测路径类型并采用相应的上传策略

    - env: 部署环境（可选）
        * "Production": 生产环境部署，使用自定义域名（如已配置）
        * "Preview": 预览环境部署，生成临时预览链接
        默认为 "Production"

    环境变量要求：
    - EDGEONE_PAGES_API_TOKEN: EdgeOne Pages API访问令牌（必需）
    - EDGEONE_PAGES_PROJECT_NAME: 项目名称（可选，未指定时自动创建临时项目）

    返回值说明：
    - status: 部署状态 ("success" 或 "error")
    - deployment_logs: 详细的部署过程日志
    - result: 部署结果信息
        * type: 域名类型 ("custom" 自定义域名 或 "temporary" 临时域名)
        * url: 网站访问URL
        * project_id: EdgeOne项目ID
        * project_name: 项目名称
        * console_url: EdgeOne控制台管理链接

    使用场景：
    1. 将本地开发的静态网站部署上线
    2. 将构建工具（如Webpack、Vite等）生成的dist目录部署
    3. 将打包好的网站ZIP文件快速部署
    4. 创建网站的预览版本进行测试

    部署流程：
    1. 验证本地路径和文件
    2. 检测可用的API端点
    3. 获取或创建EdgeOne项目
    4. 上传文件到腾讯云COS
    5. 创建部署任务并等待完成
    6. 生成访问链接和管理信息

    ⚠️ 注意事项：
    - 需要有效的EdgeOne Pages API令牌
    - 确保网络连接正常，上传可能需要一些时间
    - 大文件或大量文件的上传会相应增加部署时间
    - 临时域名链接包含时效性访问令牌
    """
    try:
        # 导入EdgeOne部署工具
        from htmlgen_mcp.agents.web_tools.edgeone_deploy import deploy_folder_or_zip_to_edgeone

        # 验证环境变量
        api_token = os.getenv("EDGEONE_PAGES_API_TOKEN")
        if not api_token:
            return {
                "status": "error",
                "message": "Missing EDGEONE_PAGES_API_TOKEN environment variable. Please set your EdgeOne Pages API token.",
            }

        # 验证路径格式
        if not os.path.isabs(folder_path):
            return {
                "status": "error",
                "message": f"Path must be absolute: {folder_path}",
            }

        # 验证环境参数
        if env not in ["Production", "Preview"]:
            return {
                "status": "error",
                "message": "env must be 'Production' or 'Preview'",
            }

        # 执行部署
        result_json = await asyncio.to_thread(
            deploy_folder_or_zip_to_edgeone, folder_path, env
        )
        result = json.loads(result_json)

        return {
            "status": "success",
            "message": f"Deployment to {env} environment completed successfully",
            "deployment_logs": result.get("deployment_logs", ""),
            "result": result.get("result", {}),
        }

    except Exception as exc:
        error_message = str(exc)

        # 如果是EdgeOne部署错误，尝试解析JSON格式的错误信息
        try:
            if error_message.startswith("{"):
                error_data = json.loads(error_message)
                return {
                    "status": "error",
                    "message": error_data.get("error", error_message),
                    "deployment_logs": error_data.get("deployment_logs", ""),
                    "traceback": traceback.format_exc(),
                }
        except:
            pass

        return {
            "status": "error",
            "message": error_message,
            "traceback": traceback.format_exc(),
        }


def main() -> None:
    transport = os.environ.get("MCP_TRANSPORT", "stdio")
    print("🚀 Smart Web Agent MCP 服务器已启动")
    print(f"📁 默认项目根目录: {DEFAULT_PROJECT_ROOT}")
    print(f"🤖 默认模型: {DEFAULT_MODEL}")
    print(f"🌐 默认API地址: {DEFAULT_BASE_URL}")
    print("🌐 EdgeOne Pages 部署工具已加载")
    mcp.run(transport=transport)


if __name__ == "__main__":
    main()



"""HTML 验证与移动端检查工具"""
from __future__ import annotations

import re


def validate_html(file_path: str):
    """简单的HTML语法验证"""
    try:
        with open(file_path, "r", encoding="utf-8") as f:
            content = f.read()

        # 基础检查
        issues = []
        if "<!DOCTYPE html>" not in content:
            issues.append("缺少DOCTYPE声明")
        if "<html" not in content:
            issues.append("缺少html标签")
        if "<head>" not in content:
            issues.append("缺少head标签")
        if "<body>" not in content:
            issues.append("缺少body标签")
        if 'charset="UTF-8"' not in content and 'charset=utf-8' not in content:
            issues.append("建议添加UTF-8字符编码")

        if issues:
            return f"HTML验证发现问题: {', '.join(issues)}"
        else:
            return "HTML结构验证通过"
    except Exception as e:
        return f"HTML验证失败: {str(e)}"


def check_mobile_friendly(file_path: str):
    """检查移动端友好性"""
    try:
        with open(file_path, "r", encoding="utf-8") as f:
            content = f.read()

        checks = []

        # 检查viewport meta标签
        if 'name="viewport"' in content:
            checks.append("✓ 包含viewport meta标签")
        else:
            checks.append("✗ 缺少viewport meta标签")

        # 检查响应式CSS
        if "@media" in content or "bootstrap" in content.lower():
            checks.append("✓ 包含响应式设计")
        else:
            checks.append("! 可能缺少响应式设计")

        # 检查图片优化
        if 'style="max-width: 100%"' in content or 'class="img-fluid"' in content:
            checks.append("✓ 图片已优化")
        else:
            checks.append("! 建议为图片添加响应式类")

        return "移动端友好性检查:\n" + "\n".join(checks)

    except Exception as e:
        return f"移动端检查失败: {str(e)}"

__all__ = ["validate_html", "check_mobile_friendly"]



"""网页工具包模块化导出"""
from __future__ import annotations

from .bootstrap import add_bootstrap
from .browser import open_in_browser, start_live_server
from .css import create_css_file
# 使用改进版的页面生成工具
try:
    from .html_templates_improved import (
        create_html_file,
        create_menu_page,
        create_about_page,
        create_contact_page,
    )
except ImportError:
    # 如果改进版不存在，回退到原版
    from .html_templates import (
        create_html_file,
        create_menu_page,
        create_about_page,
        create_contact_page,
    )
from .images import fetch_generated_images, inject_images
from .js import create_js_file
from .navigation import create_responsive_navbar
from .project import create_project_structure
from .validation import check_mobile_friendly, validate_html
# from .colors import generate_color_scheme  # 暂停导出：无需在主流程中使用

# 新增简单模板支持（暂时注释，备用组件无需默认加载）
# from .simple_templates import (
#     create_simple_html_file,
#     create_blank_html_file,
#     create_landing_page,
#     create_blog_page,
# )
# from .simple_css import (
#     create_clean_css_file,
#     create_landing_css_file,
#     create_blog_css_file,
#     create_minimal_css_file,
# )
# from .simple_js import (
#     create_simple_js_file,
#     create_minimal_js_file,
#     create_interactive_js_file,
# )
# from .simple_builder import (
#     create_simple_website,
#     create_simple_page_set,
# )
from .edgeone_deploy import (
    deploy_folder_or_zip_to_edgeone,
    EdgeOneDeployer,
)

__all__ = [
    "create_project_structure",
    "create_html_file",
    "create_menu_page",
    "create_about_page",
    "create_contact_page",
    "create_css_file",
    "create_js_file",
    "add_bootstrap",
    "create_responsive_navbar",
    "fetch_generated_images",
    "inject_images",
    "open_in_browser",
    "start_live_server",
    "validate_html",
    "check_mobile_friendly",
    # "generate_color_scheme",
    # # 简单模板
    # "create_simple_html_file",
    # "create_blank_html_file",
    # "create_landing_page",
    # "create_blog_page",
    # # 简单CSS
    # "create_clean_css_file",
    # "create_landing_css_file",
    # "create_blog_css_file",
    # "create_minimal_css_file",
    # # 简单JS
    # "create_simple_js_file",
    # "create_minimal_js_file",
    # "create_interactive_js_file",
    # # 简单网站构建器
    # "create_simple_website",
    # "create_simple_page_set",
    # EdgeOne部署工具
    "deploy_folder_or_zip_to_edgeone",
    "EdgeOneDeployer",
]



"""项目结构相关工具"""
from __future__ import annotations

from pathlib import Path


def create_project_structure(project_name: str, project_path: str) -> str:
    """创建网页项目的基础文件夹结构"""
    if project_name in project_path:
        base_path = Path(project_path)
    else:
        base_path = Path(project_path) / project_name

    dirs = [
        "assets",
        "assets/css",
        "assets/js",
        "assets/images",
    ]

    try:
        base_path.mkdir(parents=True, exist_ok=True)
        for dir_name in dirs:
            (base_path / dir_name).mkdir(parents=True, exist_ok=True)
        return f"项目结构创建成功: {base_path}"
    except Exception as exc:  # noqa: BLE001
        raise RuntimeError(f"创建项目结构失败: {str(exc)}")


__all__ = ["create_project_structure"]



"""简单网页模板工具 - 无预设内容，纯净样式"""
from __future__ import annotations

from pathlib import Path


def create_simple_html_file(
    file_path: str,
    title: str = "新页面",
    content: str = "",
    style: str = "clean"
) -> str:
    """创建简单HTML文件，无预设内容，只有基础结构"""

    # 如果没有提供内容，创建基础结构
    if not content.strip():
        content = f"""    <header>
        <h1>{title}</h1>
        <nav>
            <a href="#home">首页</a>
            <a href="#about">关于</a>
            <a href="#contact">联系</a>
        </nav>
    </header>

    <main>
        <section id="home">
            <h2>欢迎访问</h2>
            <p>这里是页面的主要内容区域。</p>
        </section>

        <section id="about">
            <h2>关于我们</h2>
            <p>在这里介绍您的内容。</p>
        </section>

        <section id="contact">
            <h2>联系方式</h2>
            <p>邮箱：your-email@example.com</p>
        </section>
    </main>

    <footer>
        <p>&copy; 2024 {title}. 保留所有权利。</p>
    </footer>"""

    html_template = f"""<!DOCTYPE html>
<html lang="zh-CN">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{title}</title>
    <link rel="stylesheet" href="assets/css/{style}.css">
</head>
<body>
{content}
</body>
</html>"""

    try:
        Path(file_path).parent.mkdir(parents=True, exist_ok=True)
        with open(file_path, "w", encoding="utf-8") as f:
            f.write(html_template)
        return f"简单HTML文件创建成功: {file_path}"
    except Exception as exc:
        raise RuntimeError(f"创建HTML文件失败: {str(exc)}")


def create_blank_html_file(file_path: str, title: str = "空白页面") -> str:
    """创建完全空白的HTML文件，只有基础结构"""

    html_template = f"""<!DOCTYPE html>
<html lang="zh-CN">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{title}</title>
    <link rel="stylesheet" href="assets/css/clean.css">
</head>
<body>
    <h1>{title}</h1>
    <p>开始编写您的内容...</p>
</body>
</html>"""

    try:
        Path(file_path).parent.mkdir(parents=True, exist_ok=True)
        with open(file_path, "w", encoding="utf-8") as f:
            f.write(html_template)
        return f"空白HTML文件创建成功: {file_path}"
    except Exception as exc:
        raise RuntimeError(f"创建HTML文件失败: {str(exc)}")


def create_landing_page(file_path: str, title: str = "着陆页", description: str = "") -> str:
    """创建简单的着陆页"""

    desc = description or "这是一个简洁的着陆页面"

    html_template = f"""<!DOCTYPE html>
<html lang="zh-CN">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{title}</title>
    <link rel="stylesheet" href="assets/css/landing.css">
</head>
<body>
    <div class="container">
        <header class="hero">
            <h1>{title}</h1>
            <p class="subtitle">{desc}</p>
            <button class="cta-button">开始使用</button>
        </header>

        <section class="features">
            <div class="feature">
                <h3>特点一</h3>
                <p>描述第一个特点</p>
            </div>
            <div class="feature">
                <h3>特点二</h3>
                <p>描述第二个特点</p>
            </div>
            <div class="feature">
                <h3>特点三</h3>
                <p>描述第三个特点</p>
            </div>
        </section>

        <footer>
            <p>联系我们：contact@example.com</p>
        </footer>
    </div>
</body>
</html>"""

    try:
        Path(file_path).parent.mkdir(parents=True, exist_ok=True)
        with open(file_path, "w", encoding="utf-8") as f:
            f.write(html_template)
        return f"着陆页创建成功: {file_path}"
    except Exception as exc:
        raise RuntimeError(f"创建着陆页失败: {str(exc)}")


def create_blog_page(file_path: str, title: str = "我的博客") -> str:
    """创建简单的博客页面"""

    html_template = f"""<!DOCTYPE html>
<html lang="zh-CN">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{title}</title>
    <link rel="stylesheet" href="assets/css/blog.css">
</head>
<body>
    <div class="container">
        <header>
            <h1>{title}</h1>
            <nav>
                <a href="#home">首页</a>
                <a href="#posts">文章</a>
                <a href="#about">关于</a>
            </nav>
        </header>

        <main>
            <article class="post">
                <h2>第一篇文章标题</h2>
                <p class="meta">发布时间：2024年1月1日</p>
                <p>这里是文章的内容。您可以在这里分享您的想法、经验或故事。</p>
            </article>

            <article class="post">
                <h2>第二篇文章标题</h2>
                <p class="meta">发布时间：2024年1月2日</p>
                <p>另一篇文章的内容。继续添加更多有趣的内容。</p>
            </article>
        </main>

        <aside class="sidebar">
            <div class="widget">
                <h3>关于作者</h3>
                <p>在这里介绍一下您自己。</p>
            </div>

            <div class="widget">
                <h3>最近文章</h3>
                <ul>
                    <li><a href="#">文章标题一</a></li>
                    <li><a href="#">文章标题二</a></li>
                    <li><a href="#">文章标题三</a></li>
                </ul>
            </div>
        </aside>

        <footer>
            <p>&copy; 2024 {title}. 保留所有权利。</p>
        </footer>
    </div>
</body>
</html>"""

    try:
        Path(file_path).parent.mkdir(parents=True, exist_ok=True)
        with open(file_path, "w", encoding="utf-8") as f:
            f.write(html_template)
        return f"博客页面创建成功: {file_path}"
    except Exception as exc:
        raise RuntimeError(f"创建博客页面失败: {str(exc)}")


__all__ = [
    "create_simple_html_file",
    "create_blank_html_file",
    "create_landing_page",
    "create_blog_page"
]


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
EdgeOne Pages 部署工具
用于将构建好的网站文件夹或ZIP文件部署到腾讯云EdgeOne Pages
"""

import os
import json
import time
import zipfile
import tempfile
from pathlib import Path
from typing import Dict, List, Optional, Any, Tuple
import requests
from dotenv import load_dotenv

# 加载环境变量
load_dotenv()

# API端点配置
BASE_API_URL1 = 'https://pages-api.cloud.tencent.com/v1'
BASE_API_URL2 = 'https://pages-api.edgeone.ai/v1'

class EdgeOneDeployError(Exception):
    """EdgeOne部署相关异常"""
    pass

class EdgeOneDeployer:
    """EdgeOne Pages 部署器"""

    def __init__(self):
        self.base_api_url = ""
        self.api_token = self._get_api_token()
        self.project_name = os.getenv('EDGEONE_PAGES_PROJECT_NAME', '')
        self.temp_project_name = None
        self.deployment_logs = []

    def _get_api_token(self) -> str:
        """获取API令牌"""
        token = os.getenv('EDGEONE_PAGES_API_TOKEN')
        if not token:
            raise EdgeOneDeployError(
                'Missing EDGEONE_PAGES_API_TOKEN. Please set it as an environment variable.'
            )
        return token

    def _get_temp_project_name(self) -> str:
        """生成临时项目名"""
        if not self.temp_project_name:
            # 基于当前工作目录生成项目名，或者如果有local_path，使用其基名
            import os
            if hasattr(self, 'local_path') and self.local_path:
                folder_name = os.path.basename(os.path.abspath(self.local_path))
            else:
                folder_name = os.path.basename(os.getcwd())

            # 清理文件夹名称，确保符合EdgeOne项目名称要求：只能包含小写字母、数字和短划线
            clean_name = ''.join(c.lower() if c.isalnum() else '-' for c in folder_name if c.isalnum() or c in '-_')
            # 移除开头和结尾的短划线，并去除连续的短划线
            clean_name = '-'.join(filter(None, clean_name.split('-')))

            if not clean_name or len(clean_name) < 3:
                clean_name = "local-upload"

            # 添加时间戳确保唯一性
            self.temp_project_name = f"{clean_name}-{int(time.time())}"
        return self.temp_project_name

    def _log(self, level: str, message: str) -> None:
        """记录日志"""
        log_entry = {
            'timestamp': time.time(),
            'level': level,
            'message': message
        }
        self.deployment_logs.append(log_entry)
        print(f"[{level}] {message}")

    async def _check_and_set_base_url(self) -> None:
        """检测并设置可用的API端点"""
        headers = {
            'Authorization': f'Bearer {self.api_token}',
            'Content-Type': 'application/json'
        }

        body = {
            'Action': 'DescribePagesProjects',
            'PageNumber': 1,
            'PageSize': 10
        }

        # 测试第一个端点
        try:
            response1 = requests.post(BASE_API_URL1, headers=headers, json=body, timeout=10)
            json1 = response1.json()
            if json1.get('Code') == 0:
                self.base_api_url = BASE_API_URL1
                self._log('INFO', 'Using BASE_API_URL1 endpoint')
                return
        except Exception:
            pass

        # 测试第二个端点
        try:
            response2 = requests.post(BASE_API_URL2, headers=headers, json=body, timeout=10)
            json2 = response2.json()
            if json2.get('Code') == 0:
                self.base_api_url = BASE_API_URL2
                self._log('INFO', 'Using BASE_API_URL2 endpoint')
                return
        except Exception:
            pass

        raise EdgeOneDeployError(
            'Invalid EDGEONE_PAGES_API_TOKEN or API endpoints unreachable. '
            'Please check your API token and network connection.'
        )

    def _make_api_request(self, action: str, data: Dict[str, Any]) -> Dict[str, Any]:
        """发起API请求"""
        headers = {
            'Authorization': f'Bearer {self.api_token}',
            'Content-Type': 'application/json'
        }

        body = {'Action': action, **data}

        try:
            response = requests.post(self.base_api_url, headers=headers, json=body, timeout=30)
            response.raise_for_status()
            return response.json()
        except requests.RequestException as e:
            raise EdgeOneDeployError(f'API request failed: {str(e)}')

    def _describe_pages_projects(self, project_id: Optional[str] = None,
                                project_name: Optional[str] = None) -> Dict[str, Any]:
        """查询Pages项目"""
        filters = []
        if project_id:
            filters.append({'Name': 'ProjectId', 'Values': [project_id]})
        if project_name:
            filters.append({'Name': 'Name', 'Values': [project_name]})

        data = {
            'Filters': filters,
            'Offset': 0,
            'Limit': 10,
            'OrderBy': 'CreatedOn'
        }

        return self._make_api_request('DescribePagesProjects', data)

    def _create_pages_project(self) -> Dict[str, Any]:
        """创建Pages项目"""
        project_name = self.project_name or self._get_temp_project_name()

        # 验证和清理项目名称，确保符合EdgeOne要求
        def validate_project_name(name: str) -> str:
            """验证并清理项目名称，确保只包含小写字母、数字和短划线"""
            if not name or name == "your_project_name":
                # 如果名称无效，使用临时名称
                return self._get_temp_project_name()

            # 清理名称：转换为小写，替换无效字符为短划线
            clean_name = ''.join(c.lower() if c.isalnum() else '-' for c in name if c.isalnum() or c in '-_')
            # 移除开头和结尾的短划线，并去除连续的短划线
            clean_name = '-'.join(filter(None, clean_name.split('-')))

            if not clean_name or len(clean_name) < 3:
                return self._get_temp_project_name()

            return clean_name

        project_name = validate_project_name(project_name)

        data = {
            'Name': project_name,
            'Provider': 'Upload',
            'Channel': 'Custom',
            'Area': 'global'
        }

        self._log('INFO', f'Creating new project: {project_name}')
        result = self._make_api_request('CreatePagesProject', data)

        # 添加调试信息以了解API响应结构
        self._log('DEBUG', f'CreatePagesProject API response: {json.dumps(result, ensure_ascii=False, indent=2)}')

        if result.get('Code') != 0:
            raise EdgeOneDeployError(f"Failed to create project: {result.get('Message', 'Unknown error')}")

        # 检查API响应中是否有错误
        response_data = result.get('Data', {}).get('Response', {})
        if 'Error' in response_data:
            error_info = response_data['Error']
            raise EdgeOneDeployError(f"API Error [{error_info.get('Code', 'Unknown')}]: {error_info.get('Message', 'Unknown error')}")

        # 尝试从不同可能的响应结构中获取项目ID
        project_id = None
        if 'Data' in result and 'Response' in result['Data']:
            response = result['Data']['Response']
            # 尝试不同的可能字段名
            project_id = response.get('ProjectId') or response.get('ProjectID') or response.get('Id') or response.get('ID')

        if not project_id:
            # 如果没有找到项目ID，抛出详细错误
            raise EdgeOneDeployError(f"ProjectId not found in API response. Response structure: {json.dumps(result, ensure_ascii=False, indent=2)}")

        self._log('INFO', f'Created project with ID: {project_id}')
        return self._describe_pages_projects(project_id=project_id)

    def _get_or_create_project(self) -> str:
        """获取或创建项目，返回项目ID"""
        if self.project_name:
            # 查找现有项目
            result = self._describe_pages_projects(project_name=self.project_name)
            projects = result['Data']['Response']['Projects']
            if projects:
                self._log('INFO', f'Project {self.project_name} already exists')
                return projects[0]['ProjectId']

        # 创建新项目
        self._log('INFO', 'Creating new project')
        result = self._create_pages_project()
        projects = result['Data']['Response']['Projects']
        if not projects:
            raise EdgeOneDeployError('Failed to retrieve project after creation')

        return projects[0]['ProjectId']

    def _get_cos_temp_token(self, project_id: Optional[str] = None) -> Dict[str, Any]:
        """获取COS临时令牌"""
        if project_id:
            data = {'ProjectId': project_id}
        else:
            data = {'ProjectName': self._get_temp_project_name()}

        result = self._make_api_request('DescribePagesCosTempToken', data)

        if result.get('Code') != 0:
            raise EdgeOneDeployError(f"Failed to get COS token: {result.get('Message', 'Unknown error')}")

        return result['Data']['Response']

    def _validate_path(self, local_path: str) -> Tuple[bool, bool]:
        """验证路径，返回(是否存在, 是否为ZIP文件)"""
        path_obj = Path(local_path)

        if not path_obj.exists():
            raise EdgeOneDeployError(f'Path does not exist: {local_path}')

        is_zip = path_obj.suffix.lower() == '.zip'

        if not path_obj.is_dir() and not is_zip:
            raise EdgeOneDeployError('Path must be a directory or ZIP file')

        return True, is_zip

    def _list_folder_files(self, folder_path: str) -> List[Dict[str, Any]]:
        """递归列出文件夹中的所有文件"""
        files = []
        folder_path = Path(folder_path)

        for item in folder_path.rglob('*'):
            if item.is_file():
                relative_path = item.relative_to(folder_path)
                files.append({
                    'path': str(item),
                    'relative_path': str(relative_path).replace('\\', '/'),
                    'size': item.stat().st_size
                })

        if len(files) > 1000000:
            raise EdgeOneDeployError('Too many files (>1M), operation cancelled')

        return files

    def _upload_to_cos(self, local_path: str, cos_config: Dict[str, Any], is_zip: bool) -> str:
        """上传文件到COS"""
        try:
            # 这里需要安装 cos-python-sdk-v5
            from qcloud_cos import CosConfig, CosS3Client
        except ImportError:
            raise EdgeOneDeployError(
                'Missing cos-python-sdk-v5 dependency. Please install: pip install cos-python-sdk-v5'
            )

        credentials = cos_config['Credentials']
        bucket = cos_config['Bucket']
        region = cos_config['Region']
        target_path = cos_config['TargetPath']

        # 初始化COS客户端
        config = CosConfig(
            Region=region,
            SecretId=credentials['TmpSecretId'],
            SecretKey=credentials['TmpSecretKey'],
            Token=credentials['Token']
        )
        client = CosS3Client(config)

        if is_zip:
            # 上传ZIP文件
            filename = os.path.basename(local_path)
            key = f"{target_path}/{filename}"

            self._log('INFO', f'Uploading ZIP file: {filename}')

            with open(local_path, 'rb') as f:
                client.put_object(
                    Bucket=bucket,
                    Body=f,
                    Key=key
                )

            self._log('INFO', 'ZIP file upload completed')
            return key
        else:
            # 上传文件夹
            files = self._list_folder_files(local_path)
            self._log('INFO', f'Uploading {len(files)} files to COS')

            for file_info in files:
                key = f"{target_path}/{file_info['relative_path']}"

                with open(file_info['path'], 'rb') as f:
                    client.put_object(
                        Bucket=bucket,
                        Body=f,
                        Key=key
                    )

            self._log('INFO', 'Folder upload completed')
            return target_path

    def _create_pages_deployment(self, project_id: str, target_path: str,
                                is_zip: bool, env: str = 'Production') -> str:
        """创建Pages部署"""
        data = {
            'ProjectId': project_id,
            'ViaMeta': 'Upload',
            'Provider': 'Upload',
            'Env': env,
            'DistType': 'Zip' if is_zip else 'Folder',
            'TempBucketPath': target_path
        }

        self._log('INFO', f'Creating deployment in {env} environment')
        result = self._make_api_request('CreatePagesDeployment', data)

        if result.get('Code') != 0:
            raise EdgeOneDeployError(f"Deployment creation failed: {result.get('Message', 'Unknown error')}")

        if result.get('Data', {}).get('Response', {}).get('Error'):
            error_msg = result['Data']['Response']['Error']['Message']
            raise EdgeOneDeployError(f"Deployment creation failed: {error_msg}")

        return result['Data']['Response']['DeploymentId']

    def _describe_pages_deployments(self, project_id: str) -> Dict[str, Any]:
        """查询Pages部署状态"""
        data = {
            'ProjectId': project_id,
            'Offset': 0,
            'Limit': 50,
            'OrderBy': 'CreatedOn',
            'Order': 'Desc'
        }

        return self._make_api_request('DescribePagesDeployments', data)

    def _poll_deployment_status(self, project_id: str, deployment_id: str) -> Dict[str, Any]:
        """轮询部署状态直到完成"""
        self._log('INFO', 'Waiting for deployment to complete')

        while True:
            result = self._describe_pages_deployments(project_id)
            deployments = result['Data']['Response']['Deployments']

            deployment = None
            for deploy in deployments:
                if deploy['DeploymentId'] == deployment_id:
                    deployment = deploy
                    break

            if not deployment:
                raise EdgeOneDeployError(f'Deployment {deployment_id} not found')

            status = deployment['Status']
            self._log('INFO', f'Deployment status: {status}')

            if status != 'Process':
                return deployment

            time.sleep(5)

    def _describe_pages_encipher_token(self, url: str) -> Dict[str, Any]:
        """获取页面访问令牌"""
        data = {'Text': url}
        return self._make_api_request('DescribePagesEncipherToken', data)

    def _get_project_console_url(self, project_id: str) -> str:
        """获取项目控制台URL"""
        if self.base_api_url == BASE_API_URL1:
            return f"https://console.cloud.tencent.com/edgeone/pages/project/{project_id}/index"
        else:
            return f"https://console.tencentcloud.com/edgeone/pages/project/{project_id}/index"

    def _format_deployment_result(self, deployment: Dict[str, Any],
                                 project_id: str, env: str = 'Production') -> Dict[str, Any]:
        """格式化部署结果"""
        if deployment['Status'] != 'Success':
            raise EdgeOneDeployError(f"Deployment failed with status: {deployment['Status']}")

        # 获取项目信息
        project_result = self._describe_pages_projects(project_id=project_id)
        projects = project_result['Data']['Response']['Projects']
        if not projects:
            raise EdgeOneDeployError('Failed to retrieve project information')

        project = projects[0]

        # 检查是否有自定义域名
        if (env == 'Production' and
            project.get('CustomDomains') and
            len(project['CustomDomains']) > 0):

            custom_domain = project['CustomDomains'][0]
            if custom_domain['Status'] == 'Pass':
                return {
                    'type': 'custom',
                    'url': f"https://{custom_domain['Domain']}",
                    'project_id': project_id,
                    'project_name': project['Name'],
                    'console_url': self._get_project_console_url(project_id)
                }

        # 使用预设域名
        domain = deployment.get('PreviewUrl', '').replace('https://', '') or project['PresetDomain']

        # 获取访问令牌
        token_result = self._describe_pages_encipher_token(domain)

        if token_result.get('Code') != 0:
            raise EdgeOneDeployError(f"Failed to get access token: {token_result.get('Message', 'Unknown error')}")

        token_data = token_result['Data']['Response']
        token = token_data['Token']
        timestamp = token_data['Timestamp']

        url = f"https://{domain}?eo_token={token}&eo_time={timestamp}"

        return {
            'type': 'temporary',
            'url': url,
            'project_id': project_id,
            'project_name': project['Name'],
            'console_url': self._get_project_console_url(project_id)
        }

    def deploy_folder_or_zip(self, local_path: str, env: str = 'Production') -> str:
        """
        部署文件夹或ZIP文件到EdgeOne Pages

        Args:
            local_path: 本地文件夹或ZIP文件路径
            env: 部署环境，'Production' 或 'Preview'

        Returns:
            部署结果的JSON字符串
        """
        try:
            # 保存local_path以供其他方法使用
            self.local_path = local_path
            self.deployment_logs = []
            self._log('INFO', f'Starting deployment of {local_path}')

            # 验证路径
            _, is_zip = self._validate_path(local_path)

            # 检查API端点
            import asyncio
            asyncio.run(self._check_and_set_base_url())

            # 获取或创建项目
            project_id = self._get_or_create_project()
            self._log('INFO', f'Using project ID: {project_id}')

            # 获取COS配置
            cos_config = self._get_cos_temp_token(project_id)

            # 上传到COS
            target_path = self._upload_to_cos(local_path, cos_config, is_zip)

            # 创建部署
            deployment_id = self._create_pages_deployment(project_id, target_path, is_zip, env)

            # 等待部署完成
            time.sleep(5)  # 等待5秒让部署开始
            deployment = self._poll_deployment_status(project_id, deployment_id)

            # 格式化结果
            result = self._format_deployment_result(deployment, project_id, env)

            # 准备返回结果
            logs_text = "\n".join([f"{log['level']}: {log['message']}" for log in self.deployment_logs])

            final_result = {
                'status': 'success',
                'deployment_logs': logs_text,
                'result': result
            }

            return json.dumps(final_result, ensure_ascii=False, indent=2)

        except Exception as e:
            # 错误处理
            logs_text = "\n".join([f"{log['level']}: {log['message']}" for log in self.deployment_logs])
            error_result = {
                'status': 'error',
                'deployment_logs': logs_text,
                'error': str(e)
            }
            raise EdgeOneDeployError(json.dumps(error_result, ensure_ascii=False, indent=2))


# 为了兼容原来的函数调用方式
def deploy_folder_or_zip_to_edgeone(local_path: str, env: str = 'Production') -> str:
    """
    部署文件夹或ZIP文件到EdgeOne Pages（函数式接口）

    Args:
        local_path: 本地文件夹或ZIP文件的绝对路径
        env: 部署环境，'Production' 或 'Preview'

    Returns:
        部署结果的JSON字符串
    """
    deployer = EdgeOneDeployer()
    return deployer.deploy_folder_or_zip(local_path, env)


"""导航栏生成工具"""
from __future__ import annotations

import html
import os
import re
from pathlib import Path


def create_responsive_navbar(file_path: str, brand_name: str = "公司名称", nav_items: list = None, cta: dict | None = None, theme: dict | None = None):
    """创建响应式导航栏组件

    参数说明：
    - file_path: 目标HTML文件路径；若非 .html 则会创建为独立组件片段。
    - brand_name: 品牌名称（导航左侧）。
    - nav_items: 导航项列表。标准结构为：
      [{"name": "首页", "href": "#home", "active": true}]
      为增强容错，也兼容以下别名键：
        • 名称：name | text | title | label
        • 链接：href | url | link
      active 缺省时默认第一个为 True，其他为 False。
    - cta: 可选字典，用于自定义右侧按钮。可包含键：
      text / short_text / href / icon / style / size_class / extra_class / visible。
      当不给出或缺少字段时，会结合导航项自动推导合适的 CTA（如"联系我们""查看菜单"等）。
    - theme: 可选字典，用于自定义导航栏主题。可包含键：
      primary_color / secondary_color / text_color / bg_opacity / use_gradient / custom_class。
      例如：{"primary_color": "#8B4513", "text_color": "white", "use_gradient": false}
    """
    # 处理主题配置
    theme = theme or {}
    primary_color = theme.get("primary_color", "#0d6efd")
    secondary_color = theme.get("secondary_color", "#6610f2")
    text_color = theme.get("text_color", "white")
    bg_opacity = theme.get("bg_opacity", "0.95")
    use_gradient = theme.get("use_gradient", True)
    custom_class = theme.get("custom_class", "navbar-glass")

    # 根据主题生成导航栏背景样式
    if use_gradient:
        navbar_bg_style = f"background: linear-gradient(135deg, {primary_color} 0%, {secondary_color} 100%);"
    else:
        # 如果primary_color是RGB格式，转换为rgba
        if primary_color.startswith("#"):
            # 简单的hex到rgba转换
            navbar_bg_style = f"background: {primary_color}; opacity: {bg_opacity};"
        elif primary_color.startswith("rgb"):
            # 将rgb转换为rgba
            navbar_bg_style = f"background: {primary_color.replace('rgb', 'rgba').replace(')', f', {bg_opacity})')};"
        else:
            navbar_bg_style = f"background: rgba({primary_color}, {bg_opacity});"

    navbar_bg_style += " backdrop-filter: blur(10px);"

    # 默认导航项
    if nav_items is None:
        nav_items = [
            {"name": "首页", "href": "#home", "active": True},
            {"name": "服务", "href": "#services", "active": False},
            {"name": "联系", "href": "#contact", "active": False}
        ]
    elif not nav_items:
        return "导航栏未生成：nav_items 为空，已跳过注入导航组件"

    # 归一化单个导航项，避免 KeyError
    def _normalize_item(raw: object, index: int) -> tuple[dict, bool]:
        explicit_active = False
        if not isinstance(raw, dict):
            return {"name": str(raw), "href": "#", "active": index == 0}, False
        name = (
            raw.get("name")
            or raw.get("text")
            or raw.get("title")
            or raw.get("label")
        )
        href = (
            raw.get("href")
            or raw.get("url")
            or raw.get("link")
        )
        explicit_active = "active" in raw
        active = raw.get("active")
        if not name:
            name = f"导航{index + 1}"
        if not href:
            # 尝试基于名称生成一个简易锚点（仅作为兜底）
            base = ''.join(ch.lower() if ch.isalnum() else '-' for ch in str(name))
            base = base.strip('-') or 'section'
            href = f"#{base}"
        if active is None:
            active = index == 0
        return {"name": name, "href": href, "active": bool(active)}, explicit_active

    normalized_items: list[dict] = []
    explicit_flags: list[bool] = []
    for i, item in enumerate(nav_items or []):
        normalized, explicit = _normalize_item(item, i)
        normalized_items.append(normalized)
        explicit_flags.append(explicit)

    # 基于当前文件自动高亮对应导航项（仅在未显式指定 active 时处理）
    current_page = os.path.basename(file_path).lower()
    if not current_page.endswith(".html"):
        current_page = ""

    highlighted = False
    for idx, item in enumerate(normalized_items):
        if explicit_flags[idx]:
            highlighted = highlighted or item.get("active", False)
            continue

        href = (item.get("href") or "").lower()
        match_current = False
        if current_page:
            if href.endswith(current_page):
                match_current = True
            elif current_page == "index.html" and href in {"index.html", "./", "#", "#home"}:
                match_current = True

        if match_current:
            item["active"] = True
            highlighted = True
        else:
            item["active"] = False

    if not highlighted and normalized_items:
        # 若仍未有激活项，默认第一个为 active
        normalized_items[0]["active"] = True

    # 生成导航项HTML
    nav_items_html = ""
    for item in normalized_items:
        active_class = " active" if item.get("active", False) else ""
        nav_items_html += (
            f'        <li class="nav-item"><a class="nav-link{active_class}" '
            f'href="{item["href"]}" style="color: {text_color};">{item["name"]}</a></li>\n'
        )

    # 品牌链接：优先跳转到第一个导航项（通常为首页），避免在子页上指向不存在的 #home
    brand_href = normalized_items[0]["href"] if normalized_items else "#home"

    def _find_item(keywords: tuple[str, ...]) -> dict | None:
        for item in normalized_items:
            name = (item.get("name") or "").lower()
            href = (item.get("href") or "").lower()
            if any(key in name for key in keywords) or any(key in href for key in keywords):
                return item
        return None

    def _derive_cta(raw_cta: dict | None) -> dict:
        # 根据主题决定CTA按钮的默认样式
        if use_gradient:
            style_default = (
                f"background: linear-gradient(135deg, {primary_color} 0%, {secondary_color} 100%);"
                f" border: none; box-shadow: 0 10px 24px rgba({int(primary_color[1:3], 16)}, {int(primary_color[3:5], 16)}, {int(primary_color[5:7], 16)}, 0.35);"
                if primary_color.startswith("#") else
                "background: linear-gradient(135deg, #0d6efd 0%, #6610f2 100%);"
                " border: none; box-shadow: 0 10px 24px rgba(13, 110, 253, 0.35);"
            )
        else:
            # 使用单色按钮，稍微变亮作为按钮色
            style_default = (
                f"background: {secondary_color or primary_color};"
                f" border: 1px solid {primary_color}; box-shadow: 0 4px 12px rgba(0, 0, 0, 0.15);"
            )

        base = {
            "text": "立即咨询",
            "short_text": "咨询",
            "href": (normalized_items[0]["href"] if normalized_items else "#home"),
            "icon": "fas fa-arrow-right",
            "style": style_default,
            "size_class": "btn-sm",
            "extra_class": "",
            "visible": True,
        }

        booking_item = _find_item(("预约", "预订", "booking", "reserve"))
        contact_item = _find_item(("联系", "contact", "call", "客服"))
        menu_item = _find_item(("menu", "菜单", "products", "服务"))
        about_item = _find_item(("about", "关于", "故事", "团队"))

        if booking_item:
            base.update({
                "text": "立即预约",
                "short_text": "预约",
                "href": booking_item.get("href", base["href"]),
                "icon": "fas fa-calendar-check",
            })
        elif contact_item:
            base.update({
                "text": "联系我们",
                "short_text": "联系",
                "href": contact_item.get("href", base["href"]),
                "icon": "fas fa-headset",
            })
        elif menu_item:
            base.update({
                "text": "查看菜单",
                "short_text": "菜单",
                "href": menu_item.get("href", base["href"]),
                "icon": "fas fa-book-open",
            })
        elif about_item:
            base.update({
                "text": "认识我们",
                "short_text": "了解",
                "href": about_item.get("href", base["href"]),
                "icon": "fas fa-user-friends",
            })

        if isinstance(raw_cta, dict):
            user_defined = {k: v for k, v in raw_cta.items() if v is not None}
            base.update(user_defined)

        return base

    def _render_cta(cfg: dict) -> str:
        if not cfg.get("visible", True):
            return ""
        text = cfg.get("text") or "立即咨询"
        short_text = cfg.get("short_text") or text
        href = cfg.get("href") or "#"
        icon_class = cfg.get("icon") or ""
        style = cfg.get("style") or (
            "background: linear-gradient(135deg, #0d6efd 0%, #6610f2 100%);"
            " border: none; box-shadow: 0 10px 24px rgba(13, 110, 253, 0.35);"
        )
        size_class = cfg.get("size_class") or "btn-sm"
        extra_class = cfg.get("extra_class") or ""
        classes = f"btn btn-gradient {size_class} rounded-pill px-3 text-white fw-bold"
        if extra_class:
            classes += f" {extra_class.strip()}"
        icon_html = f"<i class=\"{icon_class} me-1\"></i>" if icon_class else ""
        return (
            f"        <a class=\"{classes}\" href=\"{href}\" style=\"{style}\">\n"
            f"          {icon_html}<span class=\"d-none d-md-inline\">{text}</span>"
            f"<span class=\"d-md-none\">{short_text}</span>\n"
            f"        </a>\n"
        )

    cta_block = _render_cta(_derive_cta(cta))

    navbar_core = (
        f"<nav class=\"navbar navbar-expand-lg navbar-dark {custom_class}\" style=\"{navbar_bg_style}\">\n"
        f"  <div class=\"container-xxl\">\n"
        f"    <a class=\"navbar-brand d-flex align-items-center\" href=\"{brand_href}\" style=\"color: {text_color};\">\n"
        f"      <span class=\"me-2\">☕</span>{brand_name}\n"
        f"    </a>\n"
        f"    <button class=\"navbar-toggler border-0\" type=\"button\" data-bs-toggle=\"collapse\" data-bs-target=\"#navbarNav\" aria-controls=\"navbarNav\" aria-expanded=\"false\" aria-label=\"切换导航\">\n"
        f"      <span class=\"navbar-toggler-icon\"></span>\n"
        f"    </button>\n"
        f"    <div class=\"collapse navbar-collapse\" id=\"navbarNav\">\n"
        f"      <ul class=\"navbar-nav ms-auto\">\n"
        f"{nav_items_html}      </ul>\n"
        f"      <div class=\"d-flex align-items-center ms-3 gap-2\">\n"
        f"        <button class=\"theme-toggle-btn btn btn-outline-light btn-sm rounded-pill px-3\" type=\"button\" aria-label=\"切换主题\" data-action=\"toggle-theme\" style=\"border: 1px solid rgba(255,255,255,0.3); backdrop-filter: blur(10px); color: {text_color};\">\n"
        f"          <i class=\"fas fa-moon me-1\"></i><span class=\"d-none d-md-inline\">夜间</span>\n"
        f"        </button>\n"
        f"{cta_block}"
        f"      </div>\n"
        f"    </div>\n"
        f"  </div>\n"
        f"</nav>"
    )
    # 用标记包裹，便于重复调用时检测并替换，保证幂等
    navbar_html = "<!-- AI-Navbar: start -->\n" + navbar_core + "\n<!-- AI-Navbar: end -->"

    try:
        # 如果是HTML文件，直接插入导航栏
        if file_path.endswith('.html'):
            with open(file_path, "r", encoding="utf-8") as f:
                content = f.read()

            # 移除遗留的 script.js 引用，避免指向不存在的脚本
            content = re.sub(
                r"\s*<script[^>]*assets/js/script\.js[^>]*>\s*</script>",
                "",
                content,
                flags=re.I,
            )

            # 尝试清理页面顶部的“简易锚点列表导航”（ul>li>a href="#..."），避免出现两个导航
            try:
                low = content.lower()
                body_idx = low.find('<body')
                insert_start = 0
                if body_idx != -1:
                    gt = low.find('>', body_idx)
                    insert_start = gt + 1 if gt != -1 else 0
                # 仅在文首 4000 字符内查找，降低误删风险
                window_end = min(len(content), insert_start + 4000)
                head_slice = content[insert_start:window_end]

                ul_pattern = re.compile(r"<ul[^>]*>(?P<items>(?:\s*<li[^>]*>\s*<a[^>]*href=\s*['\"]#.+?['\"][^>]*>.*?</a>\s*</li>){3,}\s*)</ul>", re.I | re.S)
                candidates = list(ul_pattern.finditer(head_slice))
                for m in candidates:
                    items = m.group('items')
                    li_count = len(re.findall(r"<li\b", items, re.I))
                    anchor_count = len(re.findall(r"href=\s*['\"]#", items, re.I))
                    label_texts = [t.strip().lower() for t in re.findall(r">\s*([^<]{1,30})\s*</a>", items)]
                    common_labels = {"home", "products", "reviews", "contact", "about", "services", "案例", "产品", "联系", "关于", "首页", "服务"}
                    has_common = any(txt in common_labels for txt in label_texts)
                    # 需满足：至少3项内部锚点，且匹配常见导航标签或 ul 上 class 提示
                    ul_tag_open = head_slice[max(0, m.start()-200):m.end()]
                    class_hint = bool(re.search(r"class=\"[^\"]*(nav|menu|links)\b", ul_tag_open, re.I))
                    if li_count >= 3 and anchor_count >= 3 and (has_common or class_hint):
                        # 删除该 UL，避免重复导航
                        abs_start = insert_start + m.start()
                        abs_end = insert_start + m.end()
                        content = content[:abs_start] + content[abs_end:]
                        break
            except Exception:
                # 清理失败不影响主流程
                pass

            # 1) 已存在我们插入的导航，直接替换
            if "<!-- AI-Navbar: start -->" in content and "<!-- AI-Navbar: end -->" in content:
                content = re.sub(r"<!-- AI-Navbar: start -->.*?<!-- AI-Navbar: end -->", navbar_html, content, flags=re.S)
            else:
                # 2) 若存在任何带 .navbar 的 <nav>，替换第一个为我们的导航
                pattern = re.compile(r"<nav[^>]*class=\"[^\"]*navbar[^\"]*\"[^>]*>.*?</nav>", re.S | re.I)
                if pattern.search(content):
                    content = pattern.sub(navbar_html, content, count=1)
                else:
                    # 3) 否则在 <body...> 标签闭合后插入
                    lower = content.lower()
                    body_idx = lower.find('<body')
                    if body_idx != -1:
                        gt_idx = lower.find('>', body_idx)
                        if gt_idx != -1:
                            content = content[:gt_idx+1] + "\n    " + navbar_html + "\n" + content[gt_idx+1:]
                        else:
                            content = content.replace("<body>", f"<body>\n    {navbar_html}\n")
                    else:
                        # 没找到 body，兜底：前置插入
                        content = navbar_html + "\n" + content

            # 确保 Bootstrap 依赖存在（导航栏需要）
            bootstrap_css = '<link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/css/bootstrap.min.css" rel="stylesheet">'
            bootstrap_js = '<script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/js/bootstrap.bundle.min.js"></script>'
            fontawesome_css = '<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.4.0/css/all.min.css">'
            
            # 添加 Bootstrap CSS（如果不存在）
            if 'bootstrap' not in content.lower():
                if '</head>' in content:
                    # 在 </head> 前插入，但要在自定义 CSS 之前
                    if '<link rel="stylesheet" href="assets/css/style.css">' in content:
                        content = content.replace(
                            '<link rel="stylesheet" href="assets/css/style.css">',
                            f'{bootstrap_css}\n    {fontawesome_css}\n    <link rel="stylesheet" href="assets/css/style.css">'
                        )
                    else:
                        content = content.replace('</head>', f'    {bootstrap_css}\n    {fontawesome_css}\n</head>')
            
            # 添加 Bootstrap JS（如果不存在）
            if 'bootstrap.bundle' not in content.lower():
                if '</body>' in content:
                    # 在 </body> 前插入
                    if '<script src="assets/js/main.js"></script>' in content:
                        content = content.replace(
                            '<script src="assets/js/main.js"></script>',
                            f'{bootstrap_js}\n    <script src="assets/js/main.js"></script>'
                        )
                    else:
                        content = content.replace('</body>', f'    {bootstrap_js}\n</body>')
            
            # 确保导航锚点对应的区块存在
            id_pattern = re.compile(r'id\s*=\s*["\']([^"\']+)["\']', re.I)
            existing_ids = {m.group(1).strip().lower() for m in id_pattern.finditer(content)}
            missing_sections: list[str] = []
            for item in normalized_items:
                href = (item.get("href") or "").strip()
                if not href.startswith("#") or len(href) <= 1:
                    continue
                anchor = href[1:].strip()
                if not anchor:
                    continue
                if anchor.lower() in existing_ids:
                    continue
                heading = html.escape(item.get("name") or anchor)
                placeholder = (
                    f"\n    <section id=\"{anchor}\" class=\"section py-5 bg-light ai-nav-placeholder\">\n"
                    "      <div class=\"container text-center\">\n"
                    f"        <h2 class=\"h4 text-secondary mb-3\">{heading}</h2>\n"
                    "        <p class=\"text-muted\">此区块由导航自动生成，请替换为真实内容。</p>\n"
                    "      </div>\n"
                    "    </section>\n"
                )
                missing_sections.append(placeholder)
                existing_ids.add(anchor.lower())

            if missing_sections:
                insertion = "".join(missing_sections)
                body_close = re.search(r"</body>", content, re.I)
                if body_close:
                    idx = body_close.start()
                    content = content[:idx] + insertion + "\n" + content[idx:]
                else:
                    content = content + insertion

            with open(file_path, "w", encoding="utf-8") as f:
                f.write(content)

            return f"响应式导航栏已添加到: {file_path}"
        else:
            # 创建独立的导航栏组件文件
            Path(file_path).parent.mkdir(parents=True, exist_ok=True)
            with open(file_path, "w", encoding="utf-8") as f:
                f.write(navbar_html)
            return f"导航栏组件文件已创建: {file_path}"

    except Exception as e:
        raise RuntimeError(f"创建导航栏失败: {str(e)}")



__all__ = ["create_responsive_navbar"]
